@@ -26,6 +26,12 @@
 #include <pthread.h>
 #include <errno.h>
 #include <fcntl.h>
+
+/* ts B41126 : O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include "libburn.h"
 #include "init.h"
 #include "drive.h"
@@ -77,6 +83,8 @@ int burn_setup_drive(struct burn_drive *d, char *fname)
 	d->released = 1;
 	d->stdio_fd = -1;
 	d->status = BURN_DISC_UNREADY;
+	d->erasable = 0;
+	d->current_profile = -1;
 	d->do_stream_recording = 0;
         d->stream_recording_start= 0;
 	d->role_5_nwa = 0;
@@ -266,53 +274,7 @@ int burn_drive_inquire_media(struct burn_drive *d)
 	    (d->mdata->p2a_valid > 0 &&
 		(d->mdata->cdr_write || d->mdata->cdrw_write ||
 		 d->mdata->dvdr_write || d->mdata->dvdram_write)) ) {
-
-#define Libburn_knows_correct_state_after_loaD 1
-#ifdef Libburn_knows_correct_state_after_loaD
-
 		d->read_disc_info(d);
-
-#else
-		/* ts A61227 : This repeated read_disc_info seems
-		               to be obsoleted by above d->getcaps(d).
-		*/
-		/* ts A60908 */
-		/* Trying to stabilize the disc status after eventual load
-		   without closing and re-opening the drive */
-		/* This seems to work for burn_disc_erasable() .
-		   Speed values on RIP-14 and LITE-ON 48125S are stable
-		   and false, nevertheless. */
-		int was_equal = 0, must_equal = 3, max_loop = 20;
-		int loop_count, old_speed = -1234567890, new_speed= -987654321;
-		int old_erasable = -1234567890, new_erasable = -987654321;
-
-		fprintf(stderr,"LIBBURN_DEBUG: read_disc_info()\n");
-		for (loop_count = 0; loop_count < max_loop; loop_count++){
-			old_speed = new_speed;
-			old_erasable = new_erasable;
-
-			d->read_disc_info(d);
-			if(d->status == BURN_DISC_UNSUITABLE)
-		break;
-
-			new_speed = burn_drive_get_write_speed(d);
-			new_erasable = burn_disc_erasable(d);
-		        if (new_speed == old_speed &&
-			    new_erasable == old_erasable) {
-				was_equal++;
-				if (was_equal >= must_equal)
-		break;
-			} else
-				was_equal = 0;
-			/*
-			if (loop_count >= 1 && was_equal == 0)
-			*/
-				fprintf(stderr,"LIBBURN_DEBUG: %d : speed %d:%d   erasable %d:%d\n",
-					loop_count,old_speed,new_speed,old_erasable,new_erasable);
-			usleep(100000);
-		}
-#endif /* ! Libburn_knows_correct_state_after_loaD */
-
 	} else {
 		if (d->current_profile == -1 || d->current_is_cd_profile)
 			d->read_toc(d);
@@ -393,7 +355,7 @@ static int burn_drive__is_rdwr(char *fname, int *stat_ret,
 		if (S_ISREG(stbuf.st_mode))
 			read_size = stbuf.st_size;
 		else if (is_rdwr)
-			ret = burn_os_stdio_capacity(fname, &read_size);
+			ret = burn_os_stdio_capacity(fname, 0, &read_size);
 		if (ret <= 0 ||
 		    read_size / (off_t) 2048 >= (off_t) 0x7ffffff0) 
 			read_size = (off_t) 0x7ffffff0 * (off_t) 2048;  
@@ -450,7 +412,7 @@ int burn_drive_grab_stdio(struct burn_drive *d, int flag)
 		d->media_read_capacity =
 					read_size / 2048 - !(read_size % 2048);
 		if ((stat_ret == -1 || is_rdwr) && d->devname[0]) { 
-		       	ret = burn_os_stdio_capacity(d->devname, &size);
+		       	ret = burn_os_stdio_capacity(d->devname, 0, &size);
 			if (ret > 0)
 				burn_drive_set_media_capacity_remaining(d,
 									size);
@@ -1197,24 +1159,21 @@ void burn_drive_cancel(struct burn_drive *d)
 }
 
 
-static void strip_spaces(char *str)
+static void strip_spaces(char *str, size_t len)
 {
-	char *tmp;
+	char *tmp, *tmp2;
 
-	tmp = str + strlen(str) - 1;
-	while (isspace(*tmp))
-		*(tmp--) = '\0';
-
-	tmp = str;
-	while (*tmp) {
+	/* Remove trailing blanks */
+	for (tmp = str + len - 1; tmp >= str && (isspace(*tmp) || !*tmp); tmp--)
+		*tmp = 0;
+	/* Condense remaining blank intervals to single blanks */
+	for (tmp = str; tmp < str + len - 1 && *tmp; tmp++) {
 		if (isspace(*tmp) && isspace(*(tmp + 1))) {
-			char *tmp2;
-
-			for (tmp2 = tmp + 1; *tmp2; ++tmp2)
+			for (tmp2 = tmp + 1; tmp2 < str + len && *tmp2; tmp2++)
 				*(tmp2 - 1) = *tmp2;
 			*(tmp2 - 1) = '\0';
-		} else
-			++tmp;
+			tmp--; /* try same first blank again */
+		}
 	}
 }
 
@@ -1236,11 +1195,11 @@ static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 	id = (struct burn_scsi_inquiry_data *)d->idata;
 
 	memcpy(out->vendor, id->vendor, sizeof(id->vendor));
-	strip_spaces(out->vendor);
+	strip_spaces(out->vendor, sizeof(id->vendor));
 	memcpy(out->product, id->product, sizeof(id->product));
-	strip_spaces(out->product);
+	strip_spaces(out->product, sizeof(id->product));
 	memcpy(out->revision, id->revision, sizeof(id->revision));
-	strip_spaces(out->revision);
+	strip_spaces(out->revision, sizeof(id->revision));
 	strncpy(out->location, d->devname, 16);
 	out->location[16] = '\0';
 
@@ -1685,17 +1644,17 @@ static int burn_role_by_access(char *fname, int flag)
 #endif
 	int fd;
         
-	fd = open(fname, O_RDWR | O_LARGEFILE);
+	fd = open(fname, O_RDWR | O_LARGEFILE | O_BINARY);
 	if (fd != -1) {
 		close(fd);
 		return 2;
 	}
-	fd = open(fname, O_RDONLY | O_LARGEFILE);
+	fd = open(fname, O_RDONLY | O_LARGEFILE | O_BINARY);
         if (fd != -1) {
 		close(fd);
 		return 4;
 	}
-	fd = open(fname, O_WRONLY | O_LARGEFILE);
+	fd = open(fname, O_WRONLY | O_LARGEFILE | O_BINARY);
 	if (fd != -1) {
 		close(fd);
 		return 5;
@@ -1722,7 +1681,7 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 		is_rdwr = burn_drive__is_rdwr(fname, &stat_ret, &stbuf,
 						&read_size, 1 | 2);
 		if (stat_ret == -1 || is_rdwr) {
-			ret = burn_os_stdio_capacity(fname, &size);
+			ret = burn_os_stdio_capacity(fname, 0, &size);
 			if (ret == -1) {
 				libdax_msgs_submit(libdax_messenger, -1,
 				 0x00020009,
@@ -2637,7 +2596,8 @@ int burn_disc_get_msc1(struct burn_drive *d, int *start)
 off_t burn_disc_available_space(struct burn_drive *d,
 				 struct burn_write_opts *o)
 {
-	int lba, nwa;
+	int lba, nwa, ret;
+	off_t bytes, start_byte = 0;
 
 	if (burn_drive_is_released(d))
 		return 0;
@@ -2646,9 +2606,15 @@ off_t burn_disc_available_space(struct burn_drive *d,
 	if (d->drive_role == 0)
 		return 0;
 	if (d->drive_role != 1) {
-		if (d->media_capacity_remaining <= 0)
-			burn_drive_set_media_capacity_remaining(d,
-			  (off_t) (512 * 1024 * 1024 - 1) * (off_t) 2048);
+		if (o != NULL)
+			start_byte = o->start_byte;
+		ret = burn_os_stdio_capacity(d->devname, start_byte, &bytes);
+		if (ret != 1)
+			bytes = d->media_capacity_remaining;
+		if (bytes <= 0)
+			bytes = (off_t) (512 * 1024 * 1024 - 1) * (off_t) 2048;
+		if (bytes != d->media_capacity_remaining)
+			burn_drive_set_media_capacity_remaining(d, bytes);
 	} else {
 		if (o != NULL)
 			d->send_write_parameters(d, NULL, -1, o);
@@ -2897,7 +2863,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 		/* stdio file drive : random access read-write */
 		o->start_adr = 1;
 		size = d->media_capacity_remaining;
-		burn_os_stdio_capacity(d->devname, &size);
+		burn_os_stdio_capacity(d->devname, 0, &size);
 		burn_drive_set_media_capacity_remaining(d, size);
 		o->start_range_high = d->media_capacity_remaining;
 		o->start_alignment = 2048; /* imposting a drive, not a file */
@@ -2909,7 +2875,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 		/* stdio file drive : random access write-only */
 		o->start_adr = 1;
 		size = d->media_capacity_remaining;
-		burn_os_stdio_capacity(d->devname, &size);
+		burn_os_stdio_capacity(d->devname, 0, &size);
 		burn_drive_set_media_capacity_remaining(d, size);
 
 		/* >>> start_range_low = file size rounded to 2048 */;
