@@ -4,7 +4,8 @@
    by os.h in case of compilation for
                                   FreeBSD with CAM
 
-   Copyright (C) 2006 Thomas Schmitt <scdbackup@gmx.net>, provided under GPLv2+
+   Copyright (C) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>,
+   Provided under GPLv2+
 */
 
 /** List of all signals which shall be caught by signal handlers and trigger
@@ -14,34 +15,34 @@
 #define BURN_OS_SIGNAL_MACRO_LIST \
  SIGHUP, SIGINT, SIGQUIT, SIGILL, SIGABRT, \
  SIGFPE, SIGSEGV, SIGPIPE, SIGALRM, SIGTERM, \
- SIGUSR1, SIGUSR2, SIGXCPU, SIGTSTP, SIGTTIN, \
- SIGTTOU, \
- SIGBUS, SIGPROF, SIGSYS, SIGTRAP, \
- SIGVTALRM, SIGXCPU, SIGXFSZ 
+ SIGUSR1, SIGUSR2, SIGXCPU, SIGBUS, SIGPROF, \
+ SIGSYS, SIGTRAP, SIGVTALRM, SIGXCPU, SIGXFSZ 
 
 /* Once as text 1:1 list of strings for messages and interpreters */
 #define BURN_OS_SIGNAL_NAME_LIST \
  "SIGHUP", "SIGINT", "SIGQUIT", "SIGILL", "SIGABRT", \
  "SIGFPE", "SIGSEGV", "SIGPIPE", "SIGALRM", "SIGTERM", \
- "SIGUSR1", "SIGUSR2", "SIGXCPU", "SIGTSTP", "SIGTTIN", \
- "SIGTTOU", \
- "SIGBUS", "SIGPROF", "SIGSYS", "SIGTRAP", \
- "SIGVTALRM", "SIGXCPU", "SIGXFSZ"
+ "SIGUSR1", "SIGUSR2", "SIGXCPU", "SIGBUS", "SIGPROF", \
+ "SIGSYS", "SIGTRAP", "SIGVTALRM", "SIGXCPU", "SIGXFSZ"
 
 /* The number of above list items */
-#define BURN_OS_SIGNAL_COUNT 23
+#define BURN_OS_SIGNAL_COUNT 20
 
 /** To list all signals which shall surely not be caught */
 #define BURN_OS_NON_SIGNAL_MACRO_LIST \
-SIGKILL, SIGCHLD, SIGSTOP, SIGURG, SIGWINCH
+SIGKILL, SIGCHLD, SIGSTOP, SIGTSTP, SIGCONT, SIGTTIN, SIGTTOU, SIGURG, SIGWINCH
 
 /* The number of above list items */
-#define BURN_OS_NON_SIGNAL_COUNT 5
+#define BURN_OS_NON_SIGNAL_COUNT 9
 
 
 /* The maximum size for a (SCSI) i/o transaction */
 /* Important : MUST be at least 32768 ! */
-#define BURN_OS_TRANSPORT_BUFFER_SIZE 32768
+/* Older BSD info says that 32 kB is maximum. But 64 kB seems to work well
+   on 8-STABLE. It is by default only used with BD in streaming mode.
+   So older systems should still be quite safe with this buffer max size.
+*/
+#define BURN_OS_TRANSPORT_BUFFER_SIZE 65536
 
 
 /** To hold all state information of BSD device enumeration
