@@ -35,6 +35,11 @@ Present implementation: default dummy which enables libburn only to work
 #include <sys/statvfs.h>
 #endif /* Libburn_os_has_stavtfS */
 
+/* ts B41126 : O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include "transport.h"
 #include "drive.h"
 #include "sg.h"
@@ -244,7 +249,7 @@ int burn_os_is_2k_seekrw(char *path, int flag)
                   0 = could not estimate size capacity of file object
                   1 = estimation has been made, bytes was set
 */
-int burn_os_stdio_capacity(char *path, off_t *bytes)
+int burn_os_stdio_capacity(char *path, off_t write_start, off_t *bytes)
 {
 	struct stat stbuf;
 
@@ -277,7 +282,7 @@ int burn_os_stdio_capacity(char *path, off_t *bytes)
 		long blocks;
 
 		blocks = *bytes / 512;
-		fd = open(path, open_mode);
+		fd = open(path, open_mode | O_BINARY);
 		if (fd == -1)
 			{ret = -2; goto ex;}
 		ret = ioctl(fd, BLKGETSIZE, &blocks);
@@ -289,7 +294,7 @@ int burn_os_stdio_capacity(char *path, off_t *bytes)
 #endif /* Libburn_if_this_was_linuX */
 
 	} else if(S_ISREG(stbuf.st_mode)) {
-		add_size = stbuf.st_blocks * (off_t) 512;
+		add_size = burn_sparse_file_addsize(write_start, &stbuf);
 		strcpy(testpath, path);
 	} else
 		{ret = 0; goto ex;}
@@ -330,7 +335,7 @@ int burn_os_open_track_src(char *path, int open_flags, int flag)
 {
 	int fd;
 
-	fd = open(path, open_flags);
+	fd = open(path, open_flags | O_BINARY);
 	return fd;
 }
 
