@@ -1835,16 +1835,6 @@ try_open:;
 		d->fd = fd;
 		fcntl(fd, F_SETOWN, getpid());
 		d->released = 0;
-
-/* <<< Test for NetBSD */
-		ret = spc_confirm_cd_drive(d, 0);
-		if (ret <= 0) {
-			sg_close_drive(d);
-			d->fd = -1337;
-			return 0;
-		}
-/* <<< End Test for NetBSD */
-
 		return 1;
 	} else if (errno == EBUSY)
 		goto drive_is_in_use;
@@ -2140,6 +2130,26 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 		s.timeout = Libburn_scsi_default_timeouT;
 	if (c->page && !no_c_page) {
 		s.dxferp = c->page->data;
+
+/* # def ine Libburn_debug_dxferP 1 */
+#ifdef Libburn_debug_dxferP
+
+		{ char text[1024], *content; int i = c->page->bytes;
+			if (c->dir == FROM_DRIVE) {
+				for (i = 0; i < c->page->bytes && c->page->data[i] == 0; i++);
+				content = (i <  c->page->bytes) ?
+					  "  (some nonzero)" : "  (all zero)";
+			} else {
+				i = c->page->bytes;
+				content = "";
+			}
+			sprintf(text, "dxferp before = %lx%s",
+					(unsigned long) s.dxferp, content);
+			scsi_log_text(text, fp, 0);
+		}
+
+#endif
+
 		if (c->dir == FROM_DRIVE) {
 
 			/* ts A70519 : kernel 2.4 usb-storage seems to
@@ -2179,22 +2189,15 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 
 		c->end_time = burn_get_time(0);
 
-/*
- # de fine Libburn_ff_netbsd_mockuP
- */
-#ifdef Libburn_ff_netbsd_mockuP
-		if (c->opcode[0] == 0x5a) {
-			sprintf(msg, "Libburn_ff_netbsd_mockuP : Emulating zero reply from SCSI command 5A");
-			if (burn_sg_log_scsi & 3)
-				scsi_log_message(d, fp, msg, 0);
-			libdax_msgs_submit(libdax_messenger,
-				d->global_index, 0x00000002,
-				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
-				msg, 0, 0);
-			memset(c->page->data, 0, c->page->bytes);
-			c->error = 1;
-			c->dxfer_len = 0;
+
+#ifdef Libburn_debug_dxferP
+		if (c->page && !no_c_page) {
+			char text[1024];
+			sprintf(text, "dxferp after  = %lx",
+					(unsigned long) s.dxferp);
+			scsi_log_text(text, fp, 0);
 		}
+
 #endif
 
 		/* ts A61010 */
@@ -2381,7 +2384,7 @@ int burn_os_is_2k_seekrw(char *path, int flag)
                   0 = could not estimate size capacity of file object
                   1 = estimation has been made, bytes was set
 */
-int burn_os_stdio_capacity(char *path, off_t *bytes)
+int burn_os_stdio_capacity(char *path, off_t write_start, off_t *bytes)
 {
 	struct stat stbuf;
 	struct statvfs vfsbuf;
@@ -2414,7 +2417,7 @@ int burn_os_stdio_capacity(char *path, off_t *bytes)
 			{ret = -2; goto ex;}
 		*bytes = ((off_t) blocks) * (off_t) 512;
 	} else if(S_ISREG(stbuf.st_mode)) {
-		add_size = stbuf.st_blocks * (off_t) 512;
+		add_size = burn_sparse_file_addsize(write_start, &stbuf);
 		strcpy(testpath, path);
 	} else
 		{ret = 0; goto ex;}
