@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -261,7 +261,34 @@ void spc_request_sense(struct burn_drive *d, struct buffer *buf)
 	d->issue_command(d, c);
 }
 
-/* @return -2 = drive is ready , -1 = not ready, but no progress reported
+static int spc_report_async_error(struct burn_drive *d,
+                                  int key, int asc, int ascq, int flag)
+{
+	char *msg = NULL;
+	unsigned char sense[14];
+	int ret;
+
+	BURN_ALLOC_MEM(msg, char, BURN_DRIVE_ADR_LEN + 160);
+
+	sprintf(msg, "Asynchronous SCSI error : ");
+	sense[0] = 0x70; /* Fixed format sense data */
+	sense[2] = key;
+	sense[12] = asc;
+	sense[13] = ascq;
+	scsi_error_msg(d, sense, 14, msg + strlen(msg), &key, &asc, &ascq);
+	libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x000201a5,
+				LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+				msg, 0, 0);
+	ret = 1;
+ex:;
+	BURN_FREE_MEM(msg);
+	return ret;
+}
+
+/* @return -3 = other error reported
+           -2 = drive is ready ,
+           -1 = not ready, but no progress reported ,
            >= 0 progress indication between 0 and 65535
 */
 int spc_get_erase_progress(struct burn_drive *d)
@@ -279,6 +306,14 @@ int spc_get_erase_progress(struct burn_drive *d)
 	ret = spc_test_unit_ready_r(d, &key, &asc, &ascq, &progress);
 	if (ret > 0)
 		{ret = -2; goto ex;}
+
+	/* Check key, asc, ascq for errors other than "not yet ready" */
+	if (key != 0 &&
+	    (key != 0x2 || asc != 0x04 || ascq == 0x02 || ascq ==0x03)) {
+		spc_report_async_error(d, key, asc, ascq, 0);
+		ret= -3; goto ex;
+	}
+
 	if (progress >= 0)
 		{ret = progress; goto ex;}
 
@@ -317,6 +352,8 @@ void spc_inquiry(struct burn_drive *d)
 	c->dir = FROM_DRIVE;
 	d->issue_command(d, c);
 	id = (struct burn_scsi_inquiry_data *)d->idata;
+	id->peripheral = 0x7f; /* SPC-3: incabable undefined peripheral type */
+	id->version = 0;                  /* SPC-3: no claim for conformance */
 	memset(id->vendor, 0, 9);
 	memset(id->product, 0, 17);
 	memset(id->revision, 0, 5);
@@ -324,6 +361,8 @@ void spc_inquiry(struct burn_drive *d)
 		id->valid = -1;
 		goto ex;
 	}
+	id->peripheral = ((char *) c->page->data)[0];
+	id->version = ((char *) c->page->data)[2];
 	memcpy(id->vendor, c->page->data + 8, 8);
 	memcpy(id->product, c->page->data + 16, 16);
 	memcpy(id->revision, c->page->data + 32, 4);
@@ -367,10 +406,39 @@ void spc_allow(struct burn_drive *d)
 	d->issue_command(d, c);
 }
 
+
+/* ts B40216 : Outsourced from spc_sense_caps_al().
+               To be called by spc_sense_caps() after spc_sense_caps_al()
+*/
+static int spc_try_get_performance(struct burn_drive *d, int flag)
+{
+	int ret;
+	struct burn_feature_descr *feature_descr;
+
+	/* ts B40107 : Feature 0x107 announces availability of GET PERFORMANCE
+	               Its WSPD bit announces Type 3.
+	               Try this even if the feature is not current.
+	*/
+	ret = burn_drive_has_feature(d, 0x107, &feature_descr, 0);
+	if (ret <= 0)
+		return ret;
+	if (feature_descr->data_lenght <= 0)
+		return 1;
+	if (feature_descr->data[0] & 2)             /* WSPD */
+		ret = mmc_get_write_performance(d);
+	/* Get read performance */
+	mmc_get_performance(d, 0x00, 0);
+	return 1;
+}
+
+
 /*
 ts A70518 - A90603 : Do not call with *alloc_len < 10
 */
-/** flag&1= do only inquire alloc_len */
+/** @param flag bit0= do only inquire alloc_len
+    @return  1=ok , <=0 error ,
+             2=Block Descriptor Length > 0, retry with flag bit1
+*/
 static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 {
 	struct buffer *buf = NULL;
@@ -394,7 +462,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 
 	/* ts A90602 : Clearing mdata before command execution */
 	m = d->mdata;
-	m->valid = 0;
+	m->p2a_valid = 0;
 	burn_mdata_free_subs(m);
 
 	memset(buf, 0, sizeof(struct buffer));
@@ -411,7 +479,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	d->issue_command(d, c);
 	if (c->error) {
 		memset(buf, 0, sizeof(struct buffer));
-		m->valid = -1;
+		m->p2a_valid = -1;
 		was_error = 1;
 	}
 
@@ -420,6 +488,22 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	*/
 	block_descr_len = c->page->data[6] * 256 + c->page->data[7];
 
+	if (block_descr_len + 8 + 2 > *alloc_len) {
+		if (block_descr_len + 8 + 2 > BUFFER_SIZE || !(flag & 1)) {
+			m->p2a_valid = -1;
+			sprintf(msg,
+		 "MODE SENSE page 2A with oversized Block Descriptors: %s : %d",
+				d->devname, block_descr_len);
+			libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x0002016e, LIBDAX_MSGS_SEV_DEBUG,
+				LIBDAX_MSGS_PRIO_LOW, msg, 0, 0);
+			{ret = 0; goto ex;}
+
+		}
+		*alloc_len = block_descr_len + 10;
+		{ret = 2; goto ex;}
+	}
+
 	/* Skip over Mode Data Header and block descriptors */
 	page = c->page->data + 8 + block_descr_len;
 
@@ -438,7 +522,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	*/
 	page_length = page[1];
 	old_alloc_len = *alloc_len;
-	*alloc_len = page_length + 10;
+	*alloc_len = page_length + 10 + block_descr_len;
 	if (flag & 1)
 		{ret = !was_error; goto ex;}
 	if (page_length + 10 > old_alloc_len)
@@ -447,7 +531,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	/* ts A90602 : page_length N asserts page[N+1]. (see SPC-1 8.3.3) */
 	/* ts B11031 : qemu drive has a page_length of 18 */
 	if (page_length < 18) {
-		m->valid = -1;
+		m->p2a_valid = -1;
 		sprintf(msg, "MODE SENSE page 2A too short: %s : %d",
 			d->devname, page_length);
 		libdax_msgs_submit(libdax_messenger, d->global_index,
@@ -489,13 +573,11 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	m->max_end_lba = 0;
 
 	if (!was_error)
-		m->valid = 1;
-
-	mmc_get_configuration(d);
+		m->p2a_valid = 1;
 
 	/* ts A61225 : end of MMC-1 , begin of MMC-3 */
 	if (page_length < 30) /* no write speed descriptors ? */
-		goto try_mmc_get_performance;
+		goto no_speed_descriptors;
 
 	m->cur_write_speed = page[28] * 256 + page[29];
 
@@ -549,24 +631,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	"LIBBURN_DEBUG: 5Ah,2Ah min_write_speed = %d , max_write_speed = %d\n",
 		m->min_write_speed, m->max_write_speed);
 
-try_mmc_get_performance:;
-	if (m->cdrw_write || page_length >= 32) {
-		/* ts A90823:
-		   One has to avoid U3 enhanced memory sticks here. On my
-		   SuSE 10.2 a SanDisk Cruzer 4GB stalls at the second occasion
-		   of ACh GET PERFORMANCE. (The first one is obviously called
-		   by the OS at plug time.)
-		   This pseudo drive returns no write capabilities and a page
-		   length of 28. MMC-3 describes page length 32. Regrettably
-		   MMC-2 prescribes a page length of 26. Here i have to trust
-		   m->cdrw_write to reliably indicate any MMC-2 burner.
-		*/
-		ret = mmc_get_write_performance(d);
-		if (ret > 0 && speed_debug)
-			fprintf(stderr,
-	  "LIBBURN_DEBUG: ACh min_write_speed = %d , max_write_speed = %d\n",
-			m->min_write_speed, m->max_write_speed);
-	}
+no_speed_descriptors:;
 
 	ret = !was_error;
 ex:
@@ -585,13 +650,19 @@ void spc_sense_caps(struct burn_drive *d)
 	if (mmc_function_spy(d, "sense_caps") <= 0)
 		return;
 
+	mmc_get_configuration(d);
+
 	/* first command execution to learn Allocation Length */
 	alloc_len = start_len;
 	ret = spc_sense_caps_al(d, &alloc_len, 1);
-/*
-	fprintf(stderr,"LIBBURN_DEBUG: 5Ah alloc_len = %d , ret = %d\n",
-			alloc_len, ret);
-*/
+	if (ret == 2) {
+		/* ts B40205: Unexpectedly found Block Descriptors.
+		              Repeat with new alloc_len.
+		*/
+		ret = spc_sense_caps_al(d, &alloc_len, 1);
+		if (ret == 2)
+			goto try_get_performance;
+	}
 	/* ts B11103:
 	   qemu ATAPI DVD-ROM delivers only 28.
 	   SanDisk Cruzer U3 memory stick throws error on alloc_len < 30.
@@ -600,6 +671,9 @@ void spc_sense_caps(struct burn_drive *d)
 	if (alloc_len >= minimum_len && ret > 0)
 		/* second execution with announced length */
 		spc_sense_caps_al(d, &alloc_len, 0);
+
+try_get_performance:;
+	spc_try_get_performance(d, 0);
 }
 
 
@@ -629,7 +703,6 @@ void spc_sense_error_params(struct burn_drive *d)
 	c->page->sectors = 0;
 	c->dir = FROM_DRIVE;
 	d->issue_command(d, c);
-
 	m = d->mdata;
 	page = c->page->data + 8;
 	d->params.retries = page[3];
@@ -655,6 +728,8 @@ void spc_select_error_params(struct burn_drive *d,
 	
 	scsi_init_command(c, SPC_MODE_SELECT, sizeof(SPC_MODE_SELECT));
 	c->retry = 1;
+	if (d->mdata->retry_page_valid <= 0)
+		d->mdata->retry_page_length = 0;
 	c->opcode[8] = 8 + 2 + d->mdata->retry_page_length;
 	c->page = buf;
 	c->page->bytes = 0;
@@ -714,9 +789,11 @@ void spc_sense_write_params(struct burn_drive *d)
 	if (!c->error) {
 		page = c->page->data + 8;
 		m->write_page_length = page[1];
-		m->write_page_valid = 1;
-	} else
-		m->write_page_valid = 0;
+		if (m->write_page_length > 0)
+			m->write_page_valid = 1;
+		else
+			m->write_page_length = 0x32;
+	}
 	mmc_read_disc_info(d);
 
 	/* ts A70212 : try to setup d->media_capacity_remaining */
@@ -735,38 +812,6 @@ void spc_sense_write_params(struct burn_drive *d)
                 (double) d->media_capacity_remaining);
 */
 
-/* # def ine Libburn_test_mode_page_05_pc_01 yes */
-#ifdef Libburn_test_mode_page_05_pc_01
-
-{
-	int i;
-
-	scsi_init_command(c, SPC_MODE_SENSE, sizeof(SPC_MODE_SENSE));
-	c->dxfer_len = 8 + 2 + 0x32;
-	c->opcode[7] = (c->dxfer_len >> 8) & 0xff;
-	c->opcode[8] = c->dxfer_len & 0xff;
-	c->retry = 1;
-	c->opcode[2] = (1 << 6) | 0x05;
-	c->page = buf;
-	c->page->bytes = 0;
-	c->page->sectors = 0;
-	c->dir = FROM_DRIVE;
-	d->issue_command(d, c);
-        fprintf(stderr, "LIBBURN_DEBUG: MODE SENSE 05 PC 1 : c->error=%d\n",
-		c->error);
-	if (c->error)
-		goto ex;
-	page = c->page->data + 8;
-	for (i = 0; i < page[1] + 2 && i < 0x32 + 2; i++) {
-		fprintf(stderr, "%2.2x  ", page[i]);
-		if ((i % 10) == 9)
-			fprintf(stderr, "\n");
-	}
-	fprintf(stderr, "\n");
-}
-
-#endif /* Libburn_test_mode_page_05_pc_01 */
-
 ex:;
 	BURN_FREE_MEM(buf);
 	BURN_FREE_MEM(c);
@@ -1051,8 +1096,14 @@ int burn_scsi_setup_drive(struct burn_drive *d, int bus_no, int host_no,
 		return -1;
 	}
 	d->idata->valid = 0;
-	d->mdata->valid = 0;
+	d->mdata->p2a_valid = 0;
+	d->mdata->max_read_speed = 0;
+	d->mdata->cur_read_speed = 0;
+	d->mdata->max_write_speed = 0;
+	d->mdata->cur_write_speed = 0;
 	d->mdata->speed_descriptors = NULL;
+	d->mdata->write_page_length = 0x32;
+	d->mdata->write_page_valid = 0;
 	if (!(flag & 1)) {
 		ret = spc_setup_drive(d);
 		if (ret<=0)
@@ -1312,6 +1363,12 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 		else
 			break;
 		goto return_fail;
+	case 0x32:
+		if (*ascq == 0)
+			sprintf(msg, "No defect spare location available");
+		else
+			break;
+		goto return_fail;
 	case 0x3A:
 		if (*ascq == 0)
 			sprintf(msg, "Medium not present");
@@ -1339,6 +1396,14 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 		else
 			break;
 		goto return_fail;
+	case 0x51:
+		if (*ascq == 0)
+			sprintf(msg, "Erase failure");
+		else if (*ascq == 1)
+			sprintf(msg, "Erase failure. Incomplete erase operation");
+		else
+			break;
+		goto return_fail;
 	case 0x57:
 		if (*ascq == 0)
 			sprintf(msg, "Unable to recover Table-of-Content");
@@ -1524,7 +1589,7 @@ int scsi_notify_error(struct burn_drive *d, struct command *c,
 	int key= -1, asc= -1, ascq= -1, ret;
 	char *msg = NULL, *scsi_msg = NULL;
 
-	if (d->silent_on_scsi_error)
+	if (d->silent_on_scsi_error == 1 || d->silent_on_scsi_error == 2)
 		{ret = 1; goto ex;}
 
 	BURN_ALLOC_MEM(msg, char, 320);
@@ -1549,7 +1614,8 @@ int scsi_notify_error(struct burn_drive *d, struct command *c,
 		scsi_command_name((unsigned int) c->opcode[0], 0));
 	strcat(msg, scsi_msg);
 	ret = libdax_msgs_submit(libdax_messenger, d->global_index, 0x0002010f,
-		flag & 2 ? LIBDAX_MSGS_SEV_FAILURE : LIBDAX_MSGS_SEV_DEBUG,
+		(flag & 2) && d->silent_on_scsi_error != 3 ?
+		  LIBDAX_MSGS_SEV_FAILURE : LIBDAX_MSGS_SEV_DEBUG,
 		LIBDAX_MSGS_PRIO_HIGH, msg,0,0);
 ex:;
 	BURN_FREE_MEM(msg);
@@ -1658,6 +1724,24 @@ int scsi_log_command(unsigned char *opcode, int oplen, int data_dir,
 }
 
 
+/* ts B40731 */ 
+/* Arbitrary SCSI log message */
+int scsi_log_text(char *text, void *fp_in, int flag)
+{
+	FILE *fp = fp_in;
+
+	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
+		fprintf(fp, "%s\n", text);
+		if (burn_sg_log_scsi & 4)
+			fflush(fp);
+	}
+	if (fp == stderr || !(burn_sg_log_scsi & 2))
+		return 1;
+	fprintf(stderr, "%s\n", text);
+	return 1;
+}
+
+
 /* ts A91218 (former sg_log_cmd ts A70518) */ 
 /** Logs command (before execution) */
 int scsi_log_cmd(struct command *c, void *fp_in, int flag)
@@ -1692,7 +1776,7 @@ int scsi_log_reply(unsigned char *opcode, int data_dir, unsigned char *data,
 			l = 18;
 			if ((sense[0] & 0x7f) == 0x72 ||
 			    (sense[0] & 0x7f) == 0x73)
-				l = sense[7] + 7 + 1; /* SPC-5 4.5.2. */
+				l = sense[7] + 7 + 1; /* SPC-3 4.5.2. */
 			if (l > sense_len)
 				l = sense_len;
 			fprintf(fp, "+++ sense data =");
@@ -1744,6 +1828,22 @@ int scsi_log_err(struct burn_drive *d, struct command *c,
 	return ret;
 }
 
+/* ts B31112 */
+int scsi_log_message(struct burn_drive *d, void *fp_in, char * msg, int flag)
+{
+	int ret;
+	FILE *fp = fp_in;
+
+      	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
+		fprintf(fp, "%s\n", msg);
+		if (burn_sg_log_scsi & 4)
+			fflush(fp);
+	}
+	if (fp == stderr || !(burn_sg_log_scsi & 2))
+		return 1;
+	ret = scsi_log_message(d, stderr, msg, flag);
+	return ret;
+}
 
 /* ts B00808 */
 /*
@@ -1819,3 +1919,34 @@ ex:;
 	BURN_FREE_MEM(msg);
 	return done;
 }
+
+
+int spc_confirm_cd_drive(struct burn_drive *d, int flag)
+{
+	char *msg = NULL;
+	int ret;
+
+	BURN_ALLOC_MEM(msg, char, strlen(d->devname) + 1024); 
+
+	spc_inquiry(d);
+	if (d->idata->valid < 0) {
+		sprintf(msg, "INQUIRY failed with drive '%s'", d->devname);
+		libdax_msgs_submit(libdax_messenger, -1, 0x0002000a,
+		                   LIBDAX_MSGS_SEV_FAILURE,
+		                   LIBDAX_MSGS_PRIO_HIGH, msg, 0,0);
+		ret = 0; goto ex;
+	}
+	if (d->idata->peripheral != 0x5) {
+		sprintf(msg, "Does not identify itself as CD-ROM drive '%s'",
+		             d->devname);
+		libdax_msgs_submit(libdax_messenger, -1, 0x0002000a,
+		                   LIBDAX_MSGS_SEV_FAILURE,
+		                   LIBDAX_MSGS_PRIO_HIGH, msg, 0,0);
+		ret = 0; goto ex;
+	}
+	ret = 1;
+ex:;
+	BURN_FREE_MEM(msg);
+	return ret;
+}
+
