@@ -1,6 +1,6 @@
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -21,6 +21,11 @@
 #include <unistd.h>
 #include <errno.h>
 
+/* ts B41126 : O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include "libburn.h"
 #include "structure.h"
 #include "write.h"
@@ -65,6 +70,11 @@ struct burn_disc *burn_disc_create(void)
 	d->refcnt = 1;
 	d->sessions = 0;
 	d->session = NULL;
+
+#ifdef Libburn_disc_with_incomplete_sessioN
+	d->incomplete_sessions= 0;
+#endif
+
 	return d;
 }
 
@@ -691,10 +701,37 @@ void burn_session_get_leadout_entry(struct burn_session *s,
 
 struct burn_session **burn_disc_get_sessions(struct burn_disc *d, int *num)
 {
+
+#ifdef Libburn_disc_with_incomplete_sessioN
+
+	*num = d->sessions - d->incomplete_sessions;
+
+#else
+
 	*num = d->sessions;
+
+#endif
+
+
 	return d->session;
 }
 
+
+/* ts B30112 : API */
+int burn_disc_get_incomplete_sessions(struct burn_disc *d)
+{
+#ifdef Libburn_disc_with_incomplete_sessioN
+
+	return d->incomplete_sessions;
+
+#else
+
+	return 0;
+
+#endif
+}
+
+
 struct burn_track **burn_session_get_tracks(struct burn_session *s, int *num)
 {
 	*num = s->tracks;
@@ -737,11 +774,11 @@ int burn_disc_cd_toc_extensions(struct burn_drive *drive, int flag)
 		ret = 1;
 		goto ex;
 	}
-	track_offset = burn_session_get_start_tno(d->session[0], 0);
-	if (track_offset <= 0)
-		track_offset = 1;
 
 	for (sidx = 0; sidx < d->sessions; sidx++) {
+		track_offset = burn_session_get_start_tno(d->session[sidx], 0);
+		if (track_offset <= 0)
+			track_offset = 1;
 		if (d->session[sidx] == NULL) {
 			sprintf(msg, "d->session[%d of %d] == NULL",
 				sidx, d->sessions);
@@ -794,7 +831,9 @@ int burn_disc_cd_toc_extensions(struct burn_drive *drive, int flag)
 				if (ret > 0) {
 					ret = mmc_four_char_to_int(
 							buf->data + 24);
-					if (ret < prev_entry->track_blocks)
+					if (ret < prev_entry->track_blocks &&
+					    ((!drive->current_is_cd_profile) ||
+					   ret < prev_entry->track_blocks - 2))
 						prev_entry->track_blocks = ret;
 				}
 				prev_entry->extensions_valid |= 1;
@@ -1449,7 +1488,7 @@ static int cue_create_file_source(char *path, struct burn_cue_file_cursor *crs,
 		if (ret <= 0)
 			goto ex;
 	} else {
-		fd = open(path, O_RDONLY);
+		fd = open(path, O_RDONLY | O_BINARY);
 		if (fd == -1) {
 			sprintf(msg,
 				"In cue sheet: Cannot open FILE '%.4000s'",
