@@ -6,7 +6,7 @@
 */
 /* libburn wrappers for libisoburn 
 
-   Copyright 2007 - 2011  Thomas Schmitt, <scdbackup@gmx.net> 
+   Copyright 2007 - 2014  Thomas Schmitt, <scdbackup@gmx.net> 
    Provided under GPL version 2 or later.
 */
 
@@ -340,6 +340,9 @@ int isoburn_is_intermediate_dvd_rw(struct burn_drive *d, int flag)
                 bit5= ignore ACL from external filesystems
                 bit6= ignore POSIX Extended Attributes from external filesystems
                 bit7= pretend -ROM and scan for table of content
+                bit9= when scanning for ISO 9660 sessions on overwritable
+                      media: Do not demand a valid superblock at LBA 0
+                      and scan until end of medium.
 */ 
 static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
                                  int flag)
@@ -372,7 +375,10 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
    readonly= 1;
  if(flag & 128)
    flag = (flag & ~ 16) | 8;
-   
+
+ ret= isoburn_find_emulator(o, d, 0);
+ if(ret >= 0 && *o != NULL)
+   isoburn_destroy(o, 0);
  ret= isoburn_new(o, 0);
  if(ret<=0)
    goto ex;
@@ -410,7 +416,7 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
      }
      emulation_started= 1;
      /* try to read emulated toc */
-     ret= isoburn_emulate_toc(d, flag & 16);
+     ret= isoburn_emulate_toc(d, (flag & 16) | ((!!(flag & 512)) << 1));
      if(ret<0) {
        (*o)->emulation_mode= -1;
        goto ex;
@@ -462,7 +468,7 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
          goto ex;
        }
        emulation_started= 1;
-       ret= isoburn_emulate_toc(d, 1);
+       ret= isoburn_emulate_toc(d, 1 | ((!!(flag & 512)) << 1));
        if(ret<0)
          goto ex;
        else if(ret > 0)
@@ -481,13 +487,14 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
            goto ex;
          }
        }
-       ret= isoburn_emulate_toc(d, 1|2);
+       ret= isoburn_emulate_toc(d, 1 | 2);
        if(ret<0)
          goto ex;
        if(ret>0) { /* point msc1 to last session */
          if((*o)->toc!=NULL) {
-           for(t= (*o)->toc; t->next!=NULL; t= t->next);
-            (*o)->fabricated_msc1= t->start_lba;
+           for(t= (*o)->toc; t->next!=NULL; t= t->next)
+             ; /* clang wants newline in empty loops */
+           (*o)->fabricated_msc1= t->start_lba;
          }
        }
      }
@@ -526,6 +533,9 @@ ex:
                 bit6= ignore POSIX Extended Attributes from external filesystems
                 bit7= pretend -ROM profile and scan for table of content
                 bit8= re-assess (*drive_infos)[0] rather than aquiring adr
+                bit9= when scanning for ISO 9660 sessions on overwritable
+                      media: Do not demand a valid superblock at LBA 0
+                      and scan until end of medium.
 */
 int isoburn_drive_aquire(struct burn_drive_info *drive_infos[],
                          char *adr, int flag)
@@ -552,7 +562,7 @@ int isoburn_drive_aquire(struct burn_drive_info *drive_infos[],
    goto ex;
  drive_grabbed= 1;
  ret= isoburn_welcome_media(&o, (*drive_infos)[0].drive,
-                            (flag & (8 | 16 | 32 | 64 | 128)) | !!(flag&2));
+                         (flag & (8 | 16 | 32 | 64 | 128 | 512)) | !!(flag&2));
  if(ret<=0)
    goto ex;
 
@@ -650,6 +660,19 @@ enum burn_disc_status isoburn_disc_get_status(struct burn_drive *drive)
 }
 
 
+int isoburn_disc_pretend_full_uncond(struct burn_drive *drive)
+{
+ int ret;
+ struct isoburn *o;
+
+ ret= isoburn_find_emulator(&o, drive, 0);
+ if(ret > 0 && o != NULL)
+   o->fabricated_disc_status= BURN_DISC_FULL;
+ ret= burn_disc_pretend_full_uncond(drive);
+ return(ret);
+}
+
+
 int isoburn_disc_erasable(struct burn_drive *d)
 {
  int ret;
@@ -663,6 +686,21 @@ int isoburn_disc_erasable(struct burn_drive *d)
 }
 
 
+static int isoburn_is_overwritable(struct burn_drive *drive, int flag)
+{
+ char name[80];
+ int profile, ret;
+ 
+ ret= burn_disc_get_profile(drive, &profile, name);
+ if(ret <= 0)
+   return(0);
+ if(profile == 0x12 || profile == 0x13 || profile == 0x1a || profile == 0x43)
+   return(1);
+ return(0);
+ 
+}
+
+
 void isoburn_disc_erase(struct burn_drive *drive, int fast)
 {
  int ret, do_pseudo_blank= 0, role;
@@ -696,7 +734,9 @@ void isoburn_disc_erase(struct burn_drive *drive, int fast)
    }
    if(o->emulation_mode > 0) { /* might be readonly with emulated sessions */
      ret= burn_disc_get_multi_caps(drive, BURN_WRITE_NONE, &caps, 0);
-     if(ret > 0 && caps->start_adr)
+     if(ret <= 0) /* Maybe because of burn_disc_pretend_full() */
+       do_pseudo_blank= isoburn_is_overwritable(drive, 0); /* known profiles */
+     else if(caps->start_adr)
        do_pseudo_blank= 1;
    }
    if(do_pseudo_blank) {
@@ -1199,7 +1239,7 @@ int isoburn_report_iso_error(int iso_error_code, char msg_text[], int os_errno,
                bit15= -reserved-
    @return 1 seems to be a valid ISO image , 0 format not recognized, <0 error
 */
-int isoburn_read_iso_head_parse(struct burn_drive *d, unsigned char *data,
+int isoburn_read_iso_head_parse(unsigned char *data,
                                 int *image_blocks, char *info, int flag)
 {
  int i, info_mode;
@@ -1287,7 +1327,7 @@ int isoburn_read_iso_head(struct burn_drive *d, int lba,
      if(to_read >= (off_t) (64 * 1024))
        to_read= 64 * 1024;
      ret = burn_read_data(d, ((off_t) lba) * (off_t) 2048, (char *) buffer,
-                      to_read, &data_count, 2); /* no error messages */
+                      to_read, &data_count, 32); /* error messages as DEBUG */
    } else
      ret= 0;
    if(ret<=0)
@@ -1297,13 +1337,13 @@ int isoburn_read_iso_head(struct burn_drive *d, int lba,
  }
 
  if(flag&(1<<14)) {
-   ret= isoburn_read_iso_head_parse(d, buffer, image_blocks, info, info_mode);
+   ret= isoburn_read_iso_head_parse(buffer, image_blocks, info, info_mode);
    if(ret<0)
      goto ex;
    if(ret>0)
      {ret= 2; goto ex;}
  }
- ret= isoburn_read_iso_head_parse(d, buffer+32*1024, image_blocks, info,
+ ret= isoburn_read_iso_head_parse(buffer+32*1024, image_blocks, info,
                                   info_mode);
  if(ret<=0)
    goto ex;
@@ -1389,7 +1429,12 @@ int isoburn_emulate_toc(struct burn_drive *d, int flag)
 
  start_time= last_pacifier= time(NULL);
  lba= 0;
- if(!(flag&2)) {
+ if(flag & 2) {
+   /* If there is a PVD at LBA 32 then this is an image with emulated TOC */
+   ret= isoburn_read_iso_head(d, 32, &image_size, NULL, 0);
+   if(ret > 0)
+     lba= 32;
+ } else {
    ret= isoburn_read_iso_head(d, lba, &image_size, NULL, 0);
    if(ret<=0)
      {ret= 0; goto failure;}
@@ -1575,6 +1620,7 @@ int isoburn_toc_destroy_arrays(struct isoburn_toc_disc *o, int flag)
 struct isoburn_toc_disc *isoburn_toc_drive_get_disc(struct burn_drive *d)
 {
  int ret, session_count= 0, track_count= 0, num_tracks= 0, i, j;
+ int open_sessions= 0;
  struct isoburn *o;
  struct isoburn_toc_entry *t;
  struct isoburn_toc_disc *toc_disc= NULL;
@@ -1590,6 +1636,7 @@ struct isoburn_toc_disc *isoburn_toc_drive_get_disc(struct burn_drive *d)
  toc_disc->tracks= NULL;
  toc_disc->track_pointers= NULL;
  toc_disc->session_count= 0;
+ toc_disc->incomplete_session_count= 0;
  toc_disc->track_count= 0;
  toc_disc->toc= NULL; 
 
@@ -1630,18 +1677,26 @@ failure:;
    return(NULL);
  }
  s= burn_disc_get_sessions(toc_disc->disc, &session_count);
- for(i= 0; i<session_count; i++) {
+ open_sessions= burn_disc_get_incomplete_sessions(toc_disc->disc);
+ for(i= 0; i < session_count + open_sessions; i++) {
    tracks = burn_session_get_tracks(s[i], &num_tracks);
+   if(i == session_count + open_sessions - 1 && open_sessions > 0) {
+     /* Do not count the invisible track of the last incomplete session */
+     num_tracks--;
+   }
    track_count+= num_tracks;
  }
- if(session_count<=0 || track_count<=0)
+ if(session_count + open_sessions <= 0 || track_count <= 0)
    goto failure;
- ret= isoburn_toc_new_arrays(toc_disc, session_count, track_count, 0);
+ ret= isoburn_toc_new_arrays(toc_disc, session_count + open_sessions,
+                             track_count, 0);
  if(ret<=0)
    goto failure;
  track_count= 0;
- for(i= 0; i<session_count; i++) {
+ for(i= 0; i < session_count + open_sessions; i++) {
    tracks = burn_session_get_tracks(s[i], &num_tracks);
+   if(i == session_count + open_sessions - 1 && open_sessions > 0)
+     num_tracks--;
    toc_disc->sessions[i].session= s[i];
    toc_disc->sessions[i].track_pointers= toc_disc->track_pointers+track_count;
    toc_disc->sessions[i].track_count= num_tracks;
@@ -1653,6 +1708,7 @@ failure:;
    track_count+= num_tracks;
  }
  toc_disc->session_count= session_count;
+ toc_disc->incomplete_session_count= open_sessions;
  toc_disc->track_count= track_count;
  return(toc_disc);
 }
@@ -1661,7 +1717,7 @@ failure:;
 int isoburn_toc_disc_get_sectors(struct isoburn_toc_disc *disc)
 {
  struct isoburn_toc_entry *t;
- int ret= 0, num_sessions, num_tracks;
+ int ret= 0, num_sessions, num_tracks, open_sessions= 0, session_idx= -1;
  struct burn_session **sessions;
  struct burn_track **tracks;
  struct burn_toc_entry entry;
@@ -1673,9 +1729,23 @@ int isoburn_toc_disc_get_sectors(struct isoburn_toc_disc *disc)
      ret= t->start_lba + t->track_blocks;
  } else if(disc->disc!=NULL) {
    sessions= burn_disc_get_sessions(disc->disc, &num_sessions);
-   if(num_sessions > 0) {
-     tracks = burn_session_get_tracks(sessions[num_sessions - 1],
-                                      &num_tracks);
+   open_sessions= burn_disc_get_incomplete_sessions(disc->disc);
+   if(num_sessions + open_sessions > 0) {
+     session_idx= num_sessions + open_sessions - 1;
+     tracks = burn_session_get_tracks(sessions[session_idx], &num_tracks);
+     if(open_sessions > 0) {
+       /* Do not count the invisible track of the last incomplete session */
+       num_tracks--;
+     }
+     if(num_tracks <= 0)
+       session_idx--;
+   }
+   if(session_idx >= 0) {
+     tracks = burn_session_get_tracks(sessions[session_idx], &num_tracks);
+     if(session_idx == num_sessions + open_sessions - 1 && open_sessions > 0) {
+       /* Do not count the invisible track of the last incomplete session */
+       num_tracks--;
+     }
      if(num_tracks > 0) {
        burn_track_get_entry(tracks[num_tracks - 1], &entry);
        if(entry.extensions_valid & 1)
@@ -1698,6 +1768,12 @@ struct isoburn_toc_session **isoburn_toc_disc_get_sessions(
 }
 
 
+int isoburn_toc_disc_get_incmpl_sess(struct isoburn_toc_disc *disc)
+{
+ return(disc->incomplete_session_count);
+}
+
+
 int isoburn_toc_session_get_sectors(struct isoburn_toc_session *s)
 {
  struct isoburn_toc_entry *t;
