@@ -2,7 +2,7 @@
 /*
   Class struct of libisoburn.
 
-  Copyright 2007 - 2012 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
+  Copyright 2007 - 2015 Vreixo Formoso Lopes <metalpain2002@yahoo.es>
                     and Thomas Schmitt <scdbackup@gmx.net>
   Provided under GPL version 2 or later.
 */
@@ -676,13 +676,20 @@ struct isoburn_imgen_opts {
 
     /* Disk file paths of content of PreP partition and EFI system partition */
     char *prep_partition;
+    int prep_part_flag;
     char *efi_boot_partition;
+    int efi_boot_part_flag;
 
     /* Eventual disk file paths of prepared images which shall be appended
        after the ISO image and described by partiton table entries in a MBR.
     */
     char *appended_partitions[Libisoburn_max_appended_partitionS];
     uint8_t appended_part_types[Libisoburn_max_appended_partitionS];
+    int appended_part_flags[Libisoburn_max_appended_partitionS];
+
+    /* If 1: With appended partitions: create protective MBR and mark by GPT
+    */
+    int appended_as_gpt;
 
     /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
      */
