@@ -7,7 +7,7 @@
  See libisofs/aaip_0_2.h
      http://libburnia-project.org/wiki/AAIP
 
- Copyright (c) 2009 - 2011 Thomas Schmitt, libburnia project, GPLv2+
+ Copyright (c) 2009 - 2014 Thomas Schmitt, libburnia project, GPLv2+
 
 */
 
@@ -2065,7 +2065,7 @@ int aaip_decode_acl(unsigned char *data, size_t num_data, size_t *consumed,
  unsigned char *rpt;
  char perm_text[4], *wpt, *name= NULL;
  int type, qualifier= 0, perm, ret, cnt, name_size= 1024;
- size_t w_size, name_fill= 0, i;
+ size_t w_size= 0, name_fill= 0, i;
  uid_t uid;
  gid_t gid;
  struct passwd *pwd;
@@ -2181,6 +2181,11 @@ ex:;
 
 #include "aaip-os-freebsd.c"
 
+#else
+#ifdef __NetBSD__
+
+#include "aaip-os-freebsd.c"
+
 #else
 #ifdef __linux
 
@@ -2198,5 +2203,6 @@ ex:;
 #include "aaip-os-dummy.c"
 
 #endif /* ! __linux */
+#endif /* ! __NetBSD__ */
 #endif /* ! __FreeBSD__ */
 
