@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2012 Thomas Schmitt
+ * Copyright (c) 2009 - 2014 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -201,23 +201,26 @@ static
 int create_dir(Ecma119Image *img, IsoDir *iso, Ecma119Node **node)
 {
     int ret;
-    Ecma119Node **children;
+    Ecma119Node **children = NULL;
     struct ecma119_dir_info *dir_info;
 
-    children = calloc(1, sizeof(void*) * iso->nchildren);
-    if (children == NULL) {
-        return ISO_OUT_OF_MEM;
+    if (iso->nchildren > 0) {
+        children = calloc(1, sizeof(void*) * iso->nchildren);
+        if (children == NULL)
+            return ISO_OUT_OF_MEM;
     }
 
     dir_info = calloc(1, sizeof(struct ecma119_dir_info));
     if (dir_info == NULL) {
-        free(children);
+        if (children != NULL)
+            free(children);
         return ISO_OUT_OF_MEM;
     }
 
     ret = create_ecma119_node(img, (IsoNode*)iso, node);
     if (ret < 0) {
-        free(children);
+        if (children != NULL)
+            free(children);
         free(dir_info);
         return ret;
     }
@@ -353,7 +356,8 @@ void ecma119_node_free(Ecma119Node *node)
         for (i = 0; i < node->info.dir->nchildren; i++) {
             ecma119_node_free(node->info.dir->children[i]);
         }
-        free(node->info.dir->children);
+        if (node->info.dir->children != NULL)
+            free(node->info.dir->children);
         free(node->info.dir);
     }
     free(node->iso_name);
@@ -564,6 +568,8 @@ void sort_tree(Ecma119Node *root)
 {
     size_t i;
 
+    if (root->info.dir->children == NULL)
+        return;
     qsort(root->info.dir->children, root->info.dir->nchildren, sizeof(void*),
           cmp_node_name);
     for (i = 0; i < root->info.dir->nchildren; i++) {
@@ -592,6 +598,9 @@ int mangle_single_dir(Ecma119Image *img, Ecma119Node *dir, int max_file_len,
     nchildren = dir->info.dir->nchildren;
     children = dir->info.dir->children;
 
+    if (nchildren <= 0)
+        return ISO_SUCCESS; /* nothing to do */
+
     /* a hash table will temporary hold the names, for fast searching */
     ret = iso_htable_create((nchildren * 100) / 80, iso_str_hash,
                             (compare_function_t)strcmp, &table);
@@ -1113,6 +1122,11 @@ int family_set_ino(Ecma119Image *img, Ecma119Node **nodes, size_t family_start,
         */
         if (img_ino == prev_ino)
             img_ino = 0;
+
+	/* Accept only if it is within the 32 bit range. */
+        if (((uint64_t) img_ino) > 0xffffffff)
+            img_ino = 0;
+
     }
     if (img_ino == 0) {
         img_ino = img_give_ino_number(img->image, 0);
