@@ -206,6 +206,8 @@ void el_torito_patch_isolinux_image(ElToritoBootImage *bootimg)
 int el_torito_set_isolinux_options(ElToritoBootImage *bootimg, int options, int flag)
 {
     bootimg->isolinux_options = (options & 0x03ff);
+    bootimg->seems_boot_info_table = !!(options & 1);
+    bootimg->seems_grub2_boot_info = !!(options & (1 << 9));
     return ISO_SUCCESS;
 }
 
@@ -537,8 +539,8 @@ int iso_image_set_boot_image(IsoImage *image, const char *image_path,
     for (i = 1; i < Libisofs_max_boot_imageS; i++)
         catalog->bootimages[i] = NULL;
     catalog->node = cat_node;
-    catalog->sort_weight = 1000;                            /* slightly high */
-    if (!boot_node->explicit_weight)
+    catalog->sort_weight = 1000000000;                          /* very high */
+    if (!(boot_node->explicit_weight || boot_node->from_old_session))
         boot_node->sort_weight = 2;
     iso_node_ref((IsoNode*)cat_node);
     image->bootcat = catalog;
@@ -719,7 +721,7 @@ int iso_image_add_boot_image(IsoImage *image, const char *image_path,
     ret = create_image(image, image_path, type, &boot_img, &boot_node);
     if (ret < 0) 
         return ret;
-    if (!boot_node->explicit_weight)
+    if (!(boot_node->explicit_weight || boot_node->from_old_session))
         boot_node->sort_weight = 2;
     catalog->bootimages[catalog->num_bootimages] = boot_img;
     catalog->num_bootimages++;
@@ -812,7 +814,8 @@ write_section_header(uint8_t *buf, Ecma119Image *t, int idx, int num_entries)
         (struct el_torito_section_header *) buf;
 
     /* 0x90 = more section headers follow , 0x91 = final section */
-    e->header_indicator[0] = 0x90 + (idx == t->catalog->num_bootimages - 1);
+    e->header_indicator[0] =
+                      0x90 + (idx == t->catalog->num_bootimages - num_entries);
     e->platform_id[0] = t->catalog->bootimages[idx]->platform_id;
     e->num_entries[0] = num_entries & 0xff;
     e->num_entries[1] = (num_entries >> 8) & 0xff;;
@@ -1191,6 +1194,9 @@ int iso_patch_eltoritos(Ecma119Image *t)
         ret = iso_stream_read(original, buf, size);
         iso_stream_close(original);
         if (ret != (int) size) {
+            if (ret >= 0)
+               iso_msg_submit(t->image->id, ISO_FILE_READ_ERROR, 0,
+        "Cannot read all bytes from El Torito boot image for boot info table");
             return (ret < 0) ? ret : (int) ISO_FILE_READ_ERROR;
         }
 
