@@ -30,6 +30,11 @@
 #include <libgen.h>
 #include <string.h>
 
+/* O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 static
 int iso_file_source_new_lfs(IsoFileSource *parent, const char *name, 
                             IsoFileSource **src);
@@ -222,7 +227,7 @@ int lfs_open(IsoFileSource *src)
         data->info.dir = opendir(path);
         data->openned = data->info.dir ? 2 : 0;
     } else {
-        data->info.fd = open(path, O_RDONLY);
+        data->info.fd = open(path, O_RDONLY | O_BINARY);
         data->openned = data->info.fd != -1 ? 1 : 0;
     }
     free(path);
@@ -282,6 +287,9 @@ static
 int lfs_read(IsoFileSource *src, void *buf, size_t count)
 {
     _LocalFsFileSource *data;
+    size_t to_read, done = 0;
+    int ret;
+    uint8_t *buf8;
 
     if (src == NULL || buf == NULL) {
         return ISO_NULL_POINTER;
@@ -293,28 +301,28 @@ int lfs_read(IsoFileSource *src, void *buf, size_t count)
     data = src->data;
     switch (data->openned) {
     case 1: /* not dir */
-        {
-            int ret;
-            ret = read(data->info.fd, buf, count);
+        buf8 = (uint8_t *) buf; /* for pointer arithmetic */
+        for (to_read = count; to_read > 0; to_read = count - done) {
+            if (to_read > 1024 * 1024)
+                to_read = 1024 * 1024;
+            ret = read(data->info.fd, buf8 + done, to_read);
             if (ret < 0) {
                 /* error on read */
                 switch (errno) {
                 case EINTR:
-                    ret = ISO_INTERRUPTED;
-                    break;
+                    return ISO_INTERRUPTED;
                 case EFAULT:
-                    ret = ISO_OUT_OF_MEM;
-                    break;
+                    return ISO_OUT_OF_MEM;
                 case EIO:
-                    ret = ISO_FILE_READ_ERROR;
-                    break;
-                default:
-                    ret = ISO_FILE_ERROR;
-                    break;
+                    return ISO_FILE_READ_ERROR;
                 }
+                return ISO_FILE_ERROR;
             }
-            return ret;
+            if (ret == 0) /* EOF */
+        break;
+            done += ret;
         }
+        return done;
     case 2: /* directory */
         return ISO_FILE_IS_DIR;
     default:
@@ -514,7 +522,10 @@ int lfs_get_aa_string(IsoFileSource *src, unsigned char **aa_string, int flag)
                              &value_lengths, &values,
                              (!(flag & 2)) | 2 | (flag & 4) | 16);
     if (ret <= 0) {
-        ret = ISO_FILE_ERROR;
+        if (ret == -2)
+            ret = ISO_AAIP_NO_GET_LOCAL;
+        else
+            ret = ISO_FILE_ERROR;
         goto ex;
     }
     if (num_attrs == 0)
@@ -816,6 +827,15 @@ int iso_local_filesystem_new(IsoFilesystem **fs)
 }
 
 
+int iso_local_attr_support(int flag)
+{
+    int ret;
+
+    ret= aaip_local_attr_support(flag & 255);
+    return ret;
+}
+
+
 int iso_local_get_acl_text(char *disk_path, char **text, int flag)
 {
     int ret;
@@ -856,15 +876,19 @@ int iso_local_set_attrs(char *disk_path, size_t num_attrs, char **names,
     int ret;
 
     ret = aaip_set_attr_list(disk_path, num_attrs, names, value_lengths,
-                             values, (flag & (8 | 32)) | !(flag & 1));
+                             values, (flag & (8 | 32 | 64)) | !(flag & 1));
     if (ret <= 0) {
         if (ret == -1)
             return ISO_OUT_OF_MEM;
         if (ret == -2)
             return ISO_AAIP_BAD_AASTRING;
+        if (ret >= -5)
+            return ISO_AAIP_NO_SET_LOCAL;
         if (ret == -6 || ret == -7)
             return ISO_AAIP_NOT_ENABLED;
-        return ISO_AAIP_NO_SET_LOCAL;
+        if (ret == -8)
+            return ISO_AAIP_BAD_ATTR_NAME;
+        return ret;
     }
     return 1;
 }
