@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2011 Thomas Schmitt
+ * Copyright (c) 2009 - 2014 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -139,6 +139,7 @@ int iso_node_xinfo_dispose_cloners(int flag)
         next = assoc->next;
         free((char *) assoc);
     }
+    iso_xinfo_cloner_list= NULL;
     return(1);
 }
 
@@ -502,6 +503,30 @@ const char *iso_error_to_msg(int errcode)
         return "Too many chained symbolic links";
     case ISO_BAD_ISO_FILETYPE:
         return "Unrecognized file type in ISO image";
+    case ISO_NAME_NOT_UCS2:
+        return "Filename not suitable for character set UCS-2";
+    case ISO_IMPORT_COLLISION:
+        return "File name collision during ISO image import";
+    case ISO_HPPA_PALO_INCOMPL:
+        return "Incomplete HP-PA PALO boot parameters";
+    case ISO_HPPA_PALO_OFLOW:
+        return "HP-PA PALO boot address exceeds 2 GB";
+    case ISO_HPPA_PALO_NOTREG:
+        return "HP-PA PALO file is not a data file";
+    case ISO_HPPA_PALO_CMDLEN:
+        return "HP-PA PALO command line too long";
+    case ISO_SYSAREA_PROBLEMS:
+        return "Problems encountered during inspection of System Area";
+    case ISO_INQ_SYSAREA_PROP:
+        return "Unrecognized inquiry for system area property";
+    case ISO_ALPHA_BOOT_NOTREG:
+        return "DEC Alpha Boot Loader file is not a data file";
+    case ISO_NO_KEPT_DATA_SRC:
+        return "No data source of imported ISO image available";
+    case ISO_MALFORMED_READ_INTVL:
+        return "Malformed description string for interval reader";
+    case ISO_INTVL_READ_PROBLEM:
+        return "Unreadable file, premature EOF, or failure to seek for interval reader";
     default:
         return "Unknown error";
     }
