@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
  * Copyright (c) 2007 Mario Danic
- * Copyright (c) 2009 - 2012 Thomas Schmitt
+ * Copyright (c) 2009 - 2015 Thomas Schmitt
  * 
  * This file is part of the libisofs project; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License version 2 
@@ -32,10 +32,16 @@
 #define ISO_ROCKRIDGE_IN_DIR_REC 124
 #endif
 
+#define ISO_CE_ENTRY_SIZE  28
+
 
 static
 int susp_add_ES(Ecma119Image *t, struct susp_info *susp, int to_ce, int seqno);
 
+static
+int susp_make_CE(Ecma119Image *t, uint8_t **CE,
+                 uint32_t block_offset, uint32_t byte_offset, uint32_t size);
+
 
 static
 int susp_append(Ecma119Image *t, struct susp_info *susp, uint8_t *data)
@@ -54,14 +60,76 @@ int susp_append(Ecma119Image *t, struct susp_info *susp, uint8_t *data)
 static
 int susp_append_ce(Ecma119Image *t, struct susp_info *susp, uint8_t *data)
 {
+    int to_alloc = 1, ret;
+    unsigned char *pad;
+    uint8_t *CE;
+    size_t next_alloc;
+
+    if (data[0] &&
+        (susp->ce_len + data[2] + ISO_CE_ENTRY_SIZE - 1) / BLOCK_SIZE !=
+                                                   susp->ce_len / BLOCK_SIZE) {
+        /* Linux fs/isofs wants byte_offset + ce_len <= BLOCK_SIZE.
+           So this Continuation Area needs to end by a CE which points
+           to the start of the next block.
+        */ 
+        to_alloc = 2;
+        if ((susp->ce_len + ISO_CE_ENTRY_SIZE) % BLOCK_SIZE)
+            to_alloc = 3; /* need a PAD pseudo entry */
+    }
+
+    if (susp->ce_susp_fields == NULL)
+        susp->alloc_ce_susp_fields = 0;
+    if (susp->n_ce_susp_fields + to_alloc > susp->alloc_ce_susp_fields) {
+        next_alloc = susp->alloc_ce_susp_fields;
+        while (susp->n_ce_susp_fields + to_alloc > next_alloc)
+            next_alloc += ISO_SUSP_CE_ALLOC_STEP;
+        susp->ce_susp_fields = realloc(susp->ce_susp_fields,
+                                       sizeof(uint8_t *) * next_alloc);
+        if (susp->ce_susp_fields == NULL)
+            return ISO_OUT_OF_MEM;
+        susp->alloc_ce_susp_fields = next_alloc;
+    }
+
+    if (to_alloc >= 2) {
+        /* Insert CE entry (actual CE size later by susp_update_CE_sizes) */
+        ret = susp_make_CE(t, &CE, (uint32_t) (susp->ce_block +
+                                               susp->ce_len / BLOCK_SIZE + 1),
+                           (uint32_t) 0, (uint32_t) 2048);
+        if (ret < 0)
+            return ret;
+        susp->ce_susp_fields[susp->n_ce_susp_fields] = CE;
+        susp->ce_len += ISO_CE_ENTRY_SIZE;
+        susp->n_ce_susp_fields++;
+    }
+    if (to_alloc >= 3) {
+
+
+#ifdef Libisofs_ce_calc_debuG
+
+        fprintf(stderr,
+                  "\nlibburn_DEBUG: Inserting %d bytes of CE padding\n\n",
+                  (int) (BLOCK_SIZE - (susp->ce_len % BLOCK_SIZE)));
+
+#endif /* Libisofs_ce_calc_debuG */
+
+        pad = malloc(1);
+        if (pad == NULL)
+            return ISO_OUT_OF_MEM;
+        pad[0] = 0;
+        susp->ce_susp_fields[susp->n_ce_susp_fields] = pad;
+        if (susp->ce_len % BLOCK_SIZE)
+            susp->ce_len += BLOCK_SIZE - (susp->ce_len % BLOCK_SIZE);
+        susp->n_ce_susp_fields++;
+    }
+    susp->ce_susp_fields[susp->n_ce_susp_fields] = data;
     susp->n_ce_susp_fields++;
-    susp->ce_susp_fields = realloc(susp->ce_susp_fields, sizeof(void*)
-                                   * susp->n_ce_susp_fields);
-    if (susp->ce_susp_fields == NULL) {
-        return ISO_OUT_OF_MEM;
+
+    if (data[0] == 0) {
+        if (susp->ce_len % BLOCK_SIZE)
+            susp->ce_len += BLOCK_SIZE - (susp->ce_len % BLOCK_SIZE);
+    } else {
+        susp->ce_len += data[2];
     }
-    susp->ce_susp_fields[susp->n_ce_susp_fields - 1] = data;
-    susp->ce_len += data[2];
     return ISO_SUCCESS;
 }
 
@@ -332,6 +400,8 @@ char *get_rr_fname(Ecma119Image *t, char *str)
 
     ret = iso_get_rr_name(t->opts, t->input_charset, t->output_charset,
                           t->image->id, str, &name, 0);
+    if (ret < 0)
+        return NULL;
     return name;
 }
 
@@ -545,22 +615,56 @@ int rrip_add_SL(Ecma119Image *t, struct susp_info *susp, uint8_t **comp,
 }
 
 
+/* @param flag bit1= care about crossing block boundaries */
+static
+int susp_calc_add_to_ce(size_t *ce, size_t base_ce, int add, int flag)
+{
+    if (flag & 2) {
+        /* Account for inserted CE before size exceeds block size */
+        if ((*ce + base_ce + add + ISO_CE_ENTRY_SIZE - 1) / BLOCK_SIZE !=
+            (*ce + base_ce) / BLOCK_SIZE) {
+            /* Insert CE and padding */
+            *ce += ISO_CE_ENTRY_SIZE;
+            if ((*ce + base_ce) % BLOCK_SIZE)
+                *ce += BLOCK_SIZE - ((*ce + base_ce) % BLOCK_SIZE);
+        }
+    }
+    *ce += add;
+    return ISO_SUCCESS;
+}
+
+
 /*
-   @param flag bit0= only account sizes in sua_free resp. ce_len
-                     parameters susp and data may be NULL in this case
+   @param flag bit0= only account sizes in sua_free resp. ce_len.
+                     Parameter susp may be NULL in this case
+               bit1= account for crossing block boundaries
+                     (implied by bit0 == 0)
+   @param ce_len counts the freshly added CA size of the current node
+   @param ce_mem tells the CA size of previous nodes in the same directory
 */
 static
 int aaip_add_AL(Ecma119Image *t, struct susp_info *susp,
                 uint8_t **data, size_t num_data,
-                size_t *sua_free, size_t *ce_len, int flag)
+                size_t *sua_free, size_t *ce_len, size_t ce_mem, int flag)
 {
     int ret, done = 0, len, es_extra = 0;
     uint8_t *aapt, *cpt;
+    size_t count = 0;
 
+    if (!(flag & 1))
+        flag |= 2;
     if (!t->opts->aaip_susp_1_10)
         es_extra = 5;
     if (*sua_free < num_data + es_extra || *ce_len > 0) {
-        *ce_len += num_data + es_extra;
+        if (es_extra > 0)
+            susp_calc_add_to_ce(ce_len, ce_mem, es_extra, flag & 2);
+        done = 0;
+        for (aapt = *data; !done; aapt += aapt[2]) {
+            done = !(aapt[4] & 1);
+            len = aapt[2];
+            susp_calc_add_to_ce(ce_len, ce_mem, len, flag & 2);
+            count += len;
+        }
     } else {
         *sua_free -= num_data + es_extra;
     }
@@ -586,6 +690,7 @@ int aaip_add_AL(Ecma119Image *t, struct susp_info *susp,
     }
 
     /* Multiple fields have to be handed over as single field copies */
+    done = 0;
     for (aapt = *data; !done; aapt += aapt[2]) {
         done = !(aapt[4] & 1);
         len = aapt[2];
@@ -720,6 +825,35 @@ int aaip_add_ER(Ecma119Image *t, struct susp_info *susp, int flag)
 }
 
 
+/**
+ * Create the byte representation of a CE entry.
+ * (SUSP, 5.1).
+ */
+static
+int susp_make_CE(Ecma119Image *t, uint8_t **CE,
+                 uint32_t block_offset, uint32_t byte_offset, uint32_t size)
+{
+    uint8_t *data;
+
+    *CE = NULL;
+    data = calloc(1, 28);
+    if (data == NULL)
+        return ISO_OUT_OF_MEM;
+    *CE = data;
+
+    data[0] = 'C';
+    data[1] = 'E';
+    data[2] = 28;
+    data[3] = 1;
+
+    iso_bb(&data[4], block_offset - t->eff_partition_offset, 4);
+    iso_bb(&data[12], byte_offset, 4);
+    iso_bb(&data[20], size, 4);
+
+    return ISO_SUCCESS;
+}
+
+
 /**
  * Add a CE System Use Entry to the given tree node. A "CE" is used to add
  * a continuation area, where additional System Use Entry can be written.
@@ -728,20 +862,18 @@ int aaip_add_ER(Ecma119Image *t, struct susp_info *susp, int flag)
 static
 int susp_add_CE(Ecma119Image *t, size_t ce_len, struct susp_info *susp)
 {
-    uint8_t *CE = malloc(28);
-    if (CE == NULL) {
-        return ISO_OUT_OF_MEM;
-    }
-
-    CE[0] = 'C';
-    CE[1] = 'E';
-    CE[2] = 28;
-    CE[3] = 1;
-
-    iso_bb(&CE[4], susp->ce_block - t->eff_partition_offset, 4);
-    iso_bb(&CE[12], susp->ce_len, 4);
-    iso_bb(&CE[20], (uint32_t) ce_len, 4);
+    uint32_t block_offset, byte_offset;
+    uint8_t *CE;
+    int ret;
 
+    /* Linux fs/isofs wants byte_offset + ce_len <= BLOCK_SIZE
+     * Here the byte_offset is reduced to the minimum.
+     */
+    block_offset = susp->ce_block + susp->ce_len / BLOCK_SIZE;
+    byte_offset = susp->ce_len % BLOCK_SIZE;
+    ret = susp_make_CE(t, &CE, block_offset, byte_offset, (uint32_t) ce_len);
+    if (ret < 0)
+        return ret;
     return susp_append(t, susp, CE);
 }
 
@@ -797,6 +929,27 @@ int susp_add_ES(Ecma119Image *t, struct susp_info *susp, int to_ce, int seqno)
 }
 
 
+/** 
+ * A field beginning by 0 causes rrip_write_ce_fields() to pad up to the
+ * next block.
+ */
+static
+int pseudo_susp_add_PAD(Ecma119Image *t, struct susp_info *susp)
+{
+    unsigned char *pad;
+    int ret;
+
+    pad = malloc(1);
+    if (pad == NULL)
+        return ISO_OUT_OF_MEM;
+    pad[0] = 0;
+    ret = susp_append_ce(t, susp, pad);
+    if (ret < 0)
+        return ret;
+    return ISO_SUCCESS;
+}
+
+
 /**
  * see doc/zisofs_format.txt : "ZF System Use Entry Format"
  */
@@ -828,10 +981,12 @@ int zisofs_add_ZF(Ecma119Image *t, struct susp_info *susp, int to_ce,
 
 
 /* @param flag bit0= Do not add data but only count sua_free and ce_len
+               bit1= account for crossing block boundaries
+                     (implied by bit0 == 0)
 */
 static
 int add_zf_field(Ecma119Image *t, Ecma119Node *n, struct susp_info *info,
-                 size_t *sua_free, size_t *ce_len, int flag)
+                 size_t *sua_free, size_t *ce_len, size_t base_ce, int flag)
 {
     int ret, will_copy = 1, stream_type = 0, do_zf = 0;
     int header_size_div4 = 0, block_size_log2 = 0;
@@ -847,6 +1002,8 @@ int add_zf_field(Ecma119Image *t, Ecma119Node *n, struct susp_info *info,
                               int *header_size_div4, int *block_size_log2,
                               uint32_t *uncompressed_size, int flag);
 
+    if (!(flag & 1))
+        flag |= 2;
 
     if (iso_node_get_type(n->node) != LIBISO_FILE)
         return 2;
@@ -920,7 +1077,7 @@ int add_zf_field(Ecma119Image *t, Ecma119Node *n, struct susp_info *info,
 
     /* Account for field size */
     if (*sua_free < 16 || *ce_len > 0) {
-        *ce_len += 16;
+        susp_calc_add_to_ce(ce_len, base_ce, 16, flag & 2);
     } else {
         *sua_free -= 16;
     }
@@ -968,10 +1125,15 @@ int aaip_xinfo_cloner(void *old_data, void **new_data, int flag)
  * a CE entry of 28 bytes in SUA, this computation fails if not the 28 bytes
  * are taken into account at start. In this case the caller should retry with
  * bit0 set. 
+ * If the resulting *ce added to base_ce is in a different block than base_ce,
+ * then computation with bit0 fails and the caller should finally try bit1.
  * 
  * @param flag      bit0= assume CA usage (else return 0 on SUA overflow)
+ *                  bit1= let CA start at block start (else return 0 if
+ *                        *ce crosses a block boundary)
  * @return          1= ok, computation of *su_size and *ce is valid
  *                  0= not ok, CA usage is necessary but bit0 was not set
+ *                             or *ce crosses boundary and bit1 was not set
  *                     (*su_size and *ce stay unaltered in this case)
  *                 <0= error:
  *                 -1= not enough SUA space for 28 bytes of CE entry
@@ -979,19 +1141,44 @@ int aaip_xinfo_cloner(void *old_data, void **new_data, int flag)
  */
 static
 int susp_calc_nm_sl_al(Ecma119Image *t, Ecma119Node *n, size_t space,
-                       size_t *su_size, size_t *ce, int flag)
+                       size_t *su_size, size_t *ce, size_t base_ce, int flag)
 {
     char *name;
     size_t namelen, su_mem, ce_mem;
     void *xipt;
     size_t num_aapt = 0, sua_free = 0;
     int ret;
+    uint8_t *aapt;
+
+#ifdef Libisofs_ce_calc_debug_extrA
+
+    if (n->node->name != NULL)
+        fprintf(stderr, "libburn_DEBUG: susp_calc_nm_sl_al : %.f %s \n",
+                        (double) base_ce, n->node->name);
+
+#endif /* Libisofs_ce_calc_debug_extrA */
 
     su_mem = *su_size;
     ce_mem = *ce;
     if (*ce > 0 && !(flag & 1))
         goto unannounced_ca;
 
+    if (flag & 2) {
+        flag |= 1;
+        if (base_ce % BLOCK_SIZE) {
+
+#ifdef Libisofs_ce_calc_debuG
+
+            fprintf(stderr,
+                "\nlibburn_DEBUG: Accounting for %d bytes CE padding : %s\n\n",
+                (int) (BLOCK_SIZE - (base_ce % BLOCK_SIZE)), n->node->name);
+
+#endif /* Libisofs_ce_calc_debuG */
+
+            *ce += BLOCK_SIZE - (base_ce % BLOCK_SIZE);
+        }
+    }
+
     name = get_rr_fname(t, n->node->name);
     namelen = strlen(name);
     free(name);
@@ -1018,7 +1205,7 @@ int susp_calc_nm_sl_al(Ecma119Image *t, Ecma119Node *n, size_t space,
                 of the name will always fit into the directory entry.)
         */;
 
-        *ce = 5 + namelen;
+        susp_calc_add_to_ce(ce, base_ce, 5 + namelen, flag & 2);
         *su_size = space;
     }
     if (n->type == ECMA119_SYMLINK) {
@@ -1088,21 +1275,26 @@ int susp_calc_nm_sl_al(Ecma119Image *t, Ecma119Node *n, size_t space,
                              * the component can be divided between this
                              * and another SL entry
                              */
-                            *ce += 255; /* this SL, full */
-                            sl_len = 5 + (clen - fit);
+                            /* Will fill up old SL and write it */
+                            susp_calc_add_to_ce(ce, base_ce, 255, flag & 2);
+                            sl_len = 5 + (clen - fit); /* Start new SL */
                         } else {
                             /*
                              * the component will need a 2rd SL entry in
                              * any case, so we prefer to don't write 
                              * anything in this SL
                              */
-                            *ce += sl_len + 255;
-                            sl_len = 5 + (clen - 250) + 2;
+                            /* Will write non-full old SL */
+                            susp_calc_add_to_ce(ce, base_ce, sl_len, flag & 2);
+                            /* Will write another full SL */
+                            susp_calc_add_to_ce(ce, base_ce, 255, flag & 2);
+                            sl_len = 5 + (clen - 250) + 2; /* Start new SL */
                         }
                     } else {
                         /* case 2, create a new SL entry */
-                        *ce += sl_len;
-                        sl_len = 5 + clen;
+                        /* Will write non-full old SL */
+                        susp_calc_add_to_ce(ce, base_ce, sl_len, flag & 2);
+                        sl_len = 5 + clen; /* Start new SL */
                     }
                 } else {
                     sl_len += clen;
@@ -1124,14 +1316,14 @@ int susp_calc_nm_sl_al(Ecma119Image *t, Ecma119Node *n, size_t space,
             /* the whole SL fits into the SUA */
             *su_size += sl_len;
         } else {
-            *ce += sl_len;
+            susp_calc_add_to_ce(ce, base_ce, sl_len, flag & 2);
         }
 
     }
 
     /* Find out whether ZF is to be added and account for its bytes */
     sua_free = space - *su_size;
-    add_zf_field(t, n, NULL, &sua_free, ce, 1);
+    add_zf_field(t, n, NULL, &sua_free, ce, base_ce, 1 | (flag & 2));
     *su_size = space - sua_free;
     if (*ce > 0 && !(flag & 1))
         goto unannounced_ca;
@@ -1148,12 +1340,46 @@ int susp_calc_nm_sl_al(Ecma119Image *t, Ecma119Node *n, size_t space,
     /* let the expert decide where to add num_aapt */
     if (num_aapt > 0) {
         sua_free = space - *su_size;
-        aaip_add_AL(t, NULL, NULL, num_aapt, &sua_free, ce, 1);
+        aapt = (uint8_t *) xipt;
+        aaip_add_AL(t, NULL, &aapt, num_aapt, &sua_free, ce, base_ce,
+                    1 | (flag & 2));
         *su_size = space - sua_free;
         if (*ce > 0 && !(flag & 1))
             goto unannounced_ca;
     }
 
+#ifdef Libisofs_ce_calc_debug_filetraP
+
+    if (n->node->name != NULL)
+        if (strcmp(n->node->name, "...insert.leaf.name.here...") == 0)
+            fprintf(stderr,
+                    "libburn_DEBUG: filename breakpoint susp_calc_nm_sl_al\n");
+
+#endif /* Libisofs_ce_calc_debug_filetraP */
+
+    if (*ce > 0 && !(flag & 2)) {
+        if (base_ce / BLOCK_SIZE !=
+            (base_ce + *ce + ISO_CE_ENTRY_SIZE - 1) / BLOCK_SIZE) {
+
+#ifdef Libisofs_ce_calc_debuG
+
+            fprintf(stderr,
+    "\nlibburn_DEBUG: Crossed block boundary: %.f (%lu) -> %.f (%lu) : %s\n\n",
+                    (double) base_ce, (unsigned long) (base_ce / BLOCK_SIZE),
+                    (double) (base_ce + *ce - 1),
+                    (unsigned long)
+                        ((base_ce + *ce + ISO_CE_ENTRY_SIZE - 1) / BLOCK_SIZE),
+                     n->node->name);
+
+#endif /* Libisofs_ce_calc_debuG */
+
+            /* Crossed a block boundary */
+            *su_size = su_mem;
+            *ce = ce_mem;
+            return 0;
+        }
+    }
+
     return 1;
 
 unannounced_ca:;
@@ -1165,10 +1391,12 @@ unannounced_ca:;
 
 /* @param flag bit0= Do not add data but only count sua_free and ce_len
                      param info may be NULL in this case
+               bit1= account for crossing block boundaries
+                     (implied by bit0 == 0)
 */
 static
 int add_aa_string(Ecma119Image *t, Ecma119Node *n, struct susp_info *info,
-                  size_t *sua_free, size_t *ce_len, int flag)
+                  size_t *sua_free, size_t *ce_len, size_t base_ce, int flag)
 {
     int ret;
     uint8_t *aapt;
@@ -1183,18 +1411,20 @@ int add_aa_string(Ecma119Image *t, Ecma119Node *n, struct susp_info *info,
         num_aapt = aaip_count_bytes((unsigned char *) xipt, 0);
         if (num_aapt > 0) {
             if (flag & 1) {
-                ret = aaip_add_AL(t, NULL,NULL, num_aapt, sua_free, ce_len, 1);
+                aapt = (unsigned char *) xipt;
+                ret = aaip_add_AL(t, NULL, &aapt, num_aapt, sua_free, ce_len,
+                                  base_ce, flag & (1 | 2));
             } else {
                 aapt = malloc(num_aapt);
                 if (aapt == NULL)
                     return ISO_OUT_OF_MEM;
                 memcpy(aapt, xipt, num_aapt);
                 ret = aaip_add_AL(t, info, &aapt, num_aapt, sua_free, ce_len,
-                                  0);
+                                  base_ce, 0);
+                /* aapt is NULL now and the memory is owned by t */
             }
             if (ret < 0) 
                 return ret;
-            /* aapt is NULL now and the memory is owned by t */
         }
     }
     return 1;
@@ -1212,11 +1442,13 @@ int add_aa_string(Ecma119Image *t, Ecma119Node *n, struct susp_info *info,
  *      Already occupied space in the directory record.
  * @param ce
  *      Will be filled with the space needed in a CE
+ * @param base_ce
+ *      Predicted fill of continuation area by previous nodes of same dir
  * @return
  *      The size needed for the RR entries in the System Use Area
  */
 size_t rrip_calc_len(Ecma119Image *t, Ecma119Node *n, int type, size_t used_up,
-                     size_t *ce)
+                     size_t *ce, size_t base_ce)
 {
     size_t su_size, space;
     int ret;
@@ -1279,9 +1511,11 @@ size_t rrip_calc_len(Ecma119Image *t, Ecma119Node *n, int type, size_t used_up,
     if (type == 0) {
 
         /* Try without CE */
-        ret = susp_calc_nm_sl_al(t, n, space, &su_size, ce, 0);
-        if (ret == 0) /* Retry with CE */
-            ret = susp_calc_nm_sl_al(t, n, space, &su_size, ce, 1);
+        ret = susp_calc_nm_sl_al(t, n, space, &su_size, ce, base_ce, 0);
+        if (ret == 0) /* Retry with CE but no block crossing */
+            ret = susp_calc_nm_sl_al(t, n, space, &su_size, ce, base_ce, 1);
+        if (ret == 0) /* Retry with aligned CE and block hopping */
+            ret = susp_calc_nm_sl_al(t, n, space, &su_size, ce, base_ce, 1 | 2);
         if (ret == -2)
            return ISO_OUT_OF_MEM;
 
@@ -1308,9 +1542,13 @@ size_t rrip_calc_len(Ecma119Image *t, Ecma119Node *n, int type, size_t used_up,
             if (t->opts->aaip && !t->opts->aaip_susp_1_10) {
                 *ce += 160; /* ER of AAIP */
             }
-            /* Compute length of AAIP string of root node */
+            /* Compute length of AAIP string of root node.
+               Will write all AIIP to CA, which already starts at
+               block boundary. So no need for three tries.
+             */
             aaip_sua_free= 0;
-            ret = add_aa_string(t, n, NULL, &aaip_sua_free, &aaip_len, 1);
+            ret = add_aa_string(t, n, NULL, &aaip_sua_free, &aaip_len, base_ce,
+                                1 | 2);
             if (ret < 0)
                return ret;
             *ce += aaip_len;
@@ -1374,7 +1612,7 @@ int rrip_get_susp_fields(Ecma119Image *t, Ecma119Node *n, int type,
     size_t rrip_er_len= 182;
     size_t su_size_pd, ce_len_pd; /* predicted sizes of SUA and CA */
     int ce_is_predicted = 0;
-    size_t aaip_sua_free= 0, aaip_len= 0;
+    size_t aaip_sua_free= 0, aaip_len= 0, ce_mem;
     int space;
 
     if (t == NULL || n == NULL || info == NULL) {
@@ -1391,6 +1629,18 @@ int rrip_get_susp_fields(Ecma119Image *t, Ecma119Node *n, int type,
         return ISO_ASSERT_FAILURE;
     }
 
+    /* Mark start index of node's continuation area for later update */
+    info->current_ce_start = info->n_ce_susp_fields;
+    ce_mem = info->ce_len;
+
+#ifdef Libisofs_ce_calc_debug_filetraP
+
+    if (n->node->name != NULL)
+        if (strcmp(n->node->name, "...put.leafname.here...") == 0)
+            fprintf(stderr, "libburn_DEBUG: filename breakpoint\n");
+
+#endif /* Libisofs_ce_calc_debug_filetraP */
+
     if (type == 2 && n->parent != NULL) {
         node = n->parent;
     } else {
@@ -1500,10 +1750,14 @@ int rrip_get_susp_fields(Ecma119Image *t, Ecma119Node *n, int type,
         su_size_pd = info->suf_len;
         ce_len_pd = ce_len;
         ret = susp_calc_nm_sl_al(t, n, (size_t) space,
-                                 &su_size_pd, &ce_len_pd, 0);
+                                 &su_size_pd, &ce_len_pd, info->ce_len, 0);
         if (ret == 0) { /* Have to use CA. 28 bytes of CE are necessary */
             ret = susp_calc_nm_sl_al(t, n, (size_t) space,
-                                     &su_size_pd, &ce_len_pd, 1);
+                                     &su_size_pd, &ce_len_pd, info->ce_len, 1);
+            if (ret == 0) /* Retry with aligned CE and block hopping */
+                ret = susp_calc_nm_sl_al(t, n, (size_t) space,
+                                         &su_size_pd, &ce_len_pd, info->ce_len,
+                                         1 | 2);
             sua_free -= 28;
             ce_is_predicted = 1;
         }
@@ -1697,6 +1951,26 @@ int rrip_get_susp_fields(Ecma119Image *t, Ecma119Node *n, int type,
         }
 
         if (ce_is_predicted) {
+            if ((info->ce_len % BLOCK_SIZE) &&
+                (info->ce_len + ce_len_pd - 1 ) / BLOCK_SIZE !=
+                info->ce_len / BLOCK_SIZE) {
+                /* Linux fs/isofs wants byte_offset + ce_len <= BLOCK_SIZE
+                 * Insert padding to shift CE offset to next block start
+                 */
+
+#ifdef Libisofs_ce_calc_debuG
+
+                fprintf(stderr,
+                  "\nlibburn_DEBUG: Inserting %d bytes of CE padding : %s\n\n",
+                  (int) (BLOCK_SIZE - (info->ce_len % BLOCK_SIZE)),
+                   n->node->name);
+
+#endif /* Libisofs_ce_calc_debuG */
+
+                ret = pseudo_susp_add_PAD(t, info);
+                if (ret < 0)
+                    goto add_susp_cleanup;
+            }
             /* Add the CE entry */
             ret = susp_add_CE(t, ce_len_pd, info);
             if (ret < 0) {
@@ -1738,14 +2012,14 @@ int rrip_get_susp_fields(Ecma119Image *t, Ecma119Node *n, int type,
         }
 
         /* Eventually write zisofs ZF field */
-        ret = add_zf_field(t, n, info, &sua_free, &ce_len, 0);
+        ret = add_zf_field(t, n, info, &sua_free, &ce_len, ce_mem, 0);
         if (ret < 0)
             goto add_susp_cleanup;
 
         /* Eventually obtain AAIP field string from node
            and write it to directory entry or CE area.
         */
-        ret = add_aa_string(t, n, info, &sua_free, &ce_len, 0);
+        ret = add_aa_string(t, n, info, &sua_free, &ce_len, ce_mem, 0);
         if (ret < 0)
             goto add_susp_cleanup;
 
@@ -1806,7 +2080,8 @@ int rrip_get_susp_fields(Ecma119Image *t, Ecma119Node *n, int type,
 
             /* Compute length of AAIP string of root node */
             aaip_sua_free= 0;
-            ret = add_aa_string(t, n, NULL, &aaip_sua_free, &aaip_len, 1);
+            ret = add_aa_string(t, n, NULL, &aaip_sua_free, &aaip_len, ce_mem,
+                                1 | 2);
             if (ret < 0)
                 goto add_susp_cleanup;
 
@@ -1827,7 +2102,8 @@ int rrip_get_susp_fields(Ecma119Image *t, Ecma119Node *n, int type,
             }
             /* Write AAIP string of root node */
             aaip_sua_free= aaip_len= 0;
-            ret = add_aa_string(t, n, info, &aaip_sua_free, &aaip_len, 0);
+            ret = add_aa_string(t, n, info, &aaip_sua_free, &aaip_len, ce_mem,
+                                0);
             if (ret < 0)
                 goto add_susp_cleanup;
 
@@ -1852,6 +2128,55 @@ int rrip_get_susp_fields(Ecma119Image *t, Ecma119Node *n, int type,
     return ret;
 }
 
+
+/* Update the sizes of CE fields at end of info->susp_fields and in
+   single node range of info->ce_susp_fields.
+*/
+static
+int susp_update_CE_sizes(Ecma119Image *t, struct susp_info *info, int flag)
+{
+    size_t i, curr_pos;
+    uint8_t *curr_ce;
+    uint32_t size;
+
+    if (info->n_susp_fields == 0 ||
+        info->n_ce_susp_fields - info->current_ce_start == 0)
+        return ISO_SUCCESS;
+
+    for (i = 0; i < info->n_susp_fields; i++)
+        if (info->susp_fields[i][0] == 'C')
+            if(info->susp_fields[i][1] == 'E')
+    break;
+    if (i >= info->n_susp_fields) {
+        iso_msg_submit(t->image->id, ISO_ASSERT_FAILURE, 0,
+                       "System Use Area field contains no CE, but there are fields in Continuation Area");
+        return ISO_ASSERT_FAILURE;
+    }
+    curr_ce = info->susp_fields[i];
+    curr_pos = 0;
+    for (i = info->current_ce_start; i < info->n_ce_susp_fields; i++) {
+       if (info->ce_susp_fields[i][0] == 0) {
+           curr_pos = 0; /* pseudo SUSP PAD */
+    continue;
+       }
+       if (info->ce_susp_fields[i][0] == 'C' &&
+           info->ce_susp_fields[i][1] == 'E') {
+          size = (curr_pos + info->ce_susp_fields[i][2]) % BLOCK_SIZE;
+          if (size == 0)
+              size = BLOCK_SIZE;
+          iso_bb(curr_ce + 20, size, 4);
+          curr_ce = info->ce_susp_fields[i];
+       }
+       curr_pos = (curr_pos + info->ce_susp_fields[i][2]) % 2048;
+    }
+    if (curr_pos > 0) {
+        size = curr_pos % BLOCK_SIZE;
+        iso_bb(curr_ce + 20, size, 4);
+    }
+    return ISO_SUCCESS;
+}
+
+
 /**
  * Write the given SUSP fields into buf. Note that Continuation Area
  * fields are not written.
@@ -1864,11 +2189,16 @@ void rrip_write_susp_fields(Ecma119Image *t, struct susp_info *info,
 {
     size_t i;
     size_t pos = 0;
+    int ret;
 
     if (info->n_susp_fields == 0) {
         return;
     }
 
+    ret = susp_update_CE_sizes(t, info, 0);
+    if (ret < 0)
+        return;
+
     for (i = 0; i < info->n_susp_fields; i++) {
         memcpy(buf + pos, info->susp_fields[i], info->susp_fields[i][2]);
         pos += info->susp_fields[i][2];
@@ -1894,6 +2224,7 @@ int rrip_write_ce_fields(Ecma119Image *t, struct susp_info *info)
     size_t i;
     uint8_t *padding = NULL;
     int ret= ISO_SUCCESS;
+    uint64_t written = 0, pad_size;
 
     if (info->n_ce_susp_fields == 0) {
         goto ex;
@@ -1901,11 +2232,24 @@ int rrip_write_ce_fields(Ecma119Image *t, struct susp_info *info)
     LIBISO_ALLOC_MEM(padding, uint8_t, BLOCK_SIZE);
 
     for (i = 0; i < info->n_ce_susp_fields; i++) {
+        if (info->ce_susp_fields[i][0] == 0) {
+            /* Pseudo field: pad up to next block boundary */
+            pad_size = BLOCK_SIZE - (written % BLOCK_SIZE);
+            if (pad_size == BLOCK_SIZE)
+    continue;
+            memset(padding, 0, pad_size);
+            ret = iso_write(t, padding, pad_size);
+            if (ret < 0)
+                goto write_ce_field_cleanup;
+            written += pad_size;
+    continue;
+        }
         ret = iso_write(t, info->ce_susp_fields[i], 
                         info->ce_susp_fields[i][2]);
         if (ret < 0) {
             goto write_ce_field_cleanup;
         }
+        written += info->ce_susp_fields[i][2];
     }
 
     /* pad continuation area until block size */
@@ -1913,6 +2257,9 @@ int rrip_write_ce_fields(Ecma119Image *t, struct susp_info *info)
     if (i > 0 && i < BLOCK_SIZE) {
         memset(padding, 0, i);
         ret = iso_write(t, padding, i);
+        if (ret < 0)
+            goto write_ce_field_cleanup;
+        written += i;
     }
 
     write_ce_field_cleanup: ;
@@ -1923,6 +2270,7 @@ int rrip_write_ce_fields(Ecma119Image *t, struct susp_info *info)
     free(info->ce_susp_fields);
     info->ce_susp_fields = NULL;
     info->n_ce_susp_fields = 0;
+    info->alloc_ce_susp_fields = 0;
     info->ce_len = 0;
 ex:;
     LIBISO_FREE_MEM(padding);
