@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2011 Thomas Schmitt
+ * Copyright (c) 2011 - 2014 Thomas Schmitt
  * 
  * This file is part of the libisofs project; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License version 2 
@@ -23,6 +23,7 @@
 #include "builder.h"
 #include "messages.h"
 #include "tree.h"
+#include "util.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -502,10 +503,10 @@ int iso_tree_add_node_builder(IsoImage *image, IsoDir *parent,
     int result;
     IsoNode *new;
     IsoNode **pos;
-    char *name;
+    char *name = NULL;
 
     if (parent == NULL || src == NULL || builder == NULL) {
-        return ISO_NULL_POINTER;
+        result = ISO_NULL_POINTER; goto ex;
     }
     if (node) {
         *node = NULL;
@@ -515,23 +516,25 @@ int iso_tree_add_node_builder(IsoImage *image, IsoDir *parent,
 
     /* find place where to insert */
     result = iso_dir_exists(parent, name, &pos);
-    free(name);
     if (result) {
         /* a node with same name already exists */
-        return ISO_NODE_NAME_NOT_UNIQUE;
+        result = ISO_NODE_NAME_NOT_UNIQUE; goto ex;
     }
 
-    result = builder->create_node(builder, image, src, &new);
-    if (result < 0) {
-        return result;
-    }
+    result = builder->create_node(builder, image, src, name, &new);
+    if (result < 0)
+        goto ex;
 
     if (node) {
         *node = new;
     }
 
     /* finally, add node to parent */
-    return iso_dir_insert(parent, (IsoNode*)new, pos, ISO_REPLACE_NEVER);
+    result = iso_dir_insert(parent, (IsoNode*)new, pos, ISO_REPLACE_NEVER);
+ex:
+    if (name != NULL)
+        free(name);
+    return result;
 }
 
 int iso_tree_add_node(IsoImage *image, IsoDir *parent, const char *path,
@@ -587,7 +590,8 @@ int iso_tree_add_new_node(IsoImage *image, IsoDir *parent, const char *name,
         return result;
     }
 
-    result = image->builder->create_node(image->builder, image, file, &new);
+    result = image->builder->create_node(image->builder, image, file,
+                                         (char *) name, &new);
     
     /* free the file */
     iso_file_source_unref(file);
@@ -596,12 +600,6 @@ int iso_tree_add_new_node(IsoImage *image, IsoDir *parent, const char *name,
         return result;
     }
     
-    result = iso_node_set_name(new, name);
-    if (result < 0) {
-        iso_node_unref(new);
-        return result;
-    }
-
     if (node) {
         *node = new;
     }
@@ -742,6 +740,176 @@ int check_special(IsoImage *image, mode_t mode)
     return 0;
 }
 
+
+static
+void ascii_increment(char *name, int len, int pos, int rollover_carry)
+{
+     int c;
+
+again:;
+     if (pos < 0 || pos >= len)
+         pos = len - 1;
+     c = name[pos];
+     if (c >= '0' && c < '9') {
+         c++;
+     } else if (c == '9') {
+         c = 'A';
+     } else if (c >= 'A' && c < 'Z') {
+         c++;
+     } else if (c == 'Z') {
+         c = '_';
+     } else if (c == '_') {
+         c = 'a';
+     } else if (c >= 'a' && c < 'z') {
+         c++;
+     } else if (c == 'z') {
+         c = '0';
+         name[pos] = c;
+         pos--;
+         if (pos >= 0 || rollover_carry)
+             goto again;
+         return;
+     } else {
+         if (pos == len - 1 || name[pos + 1] == '.')
+             c = '_'; /* Make first change less riddling */
+         else
+             c = '0'; /* But else use the full range of valid characters */
+     }
+     name[pos] = c;
+}
+
+static
+int insert_underscores(char *name, int *len, int *at_pos, int count,
+                       char **new_name)
+{
+    int ret;
+
+    LIBISO_ALLOC_MEM(*new_name, char, count + *len + 1);
+    if (*at_pos > 0)
+        memcpy(*new_name, name, *at_pos);
+    if (count > 0)
+        memset(*new_name + *at_pos, '_', count);
+    if (*len > *at_pos)
+        memcpy(*new_name + *at_pos + count, name + *at_pos, *len - *at_pos);
+    (*new_name)[count + *len] = 0;
+    *len += count;
+    *at_pos += count;
+    ret= ISO_SUCCESS;
+ex:;
+    return ret;
+}
+
+static
+int make_incrementable_name(char **name, char **unique_name, int *low_pos,
+                            int *rollover_carry, int *pre_check) 
+{
+    char *dpt, *npt;
+    int first, len, ret;
+    
+    /* The incrementable part of the file shall have at least 7 characters.
+       There may be up to pow(2.0,32.0)*2048/33 = 266548273400 files.
+       The set of increment result characters has 63 elements.
+       pow(63.0,7.0) is nearly 15 times larger than 266548273400.
+    */
+    static int min_incr = 7;
+
+    /* At most two suffixes of total length up to 12, like .tar.bz2,
+       shall be preserved. The incrementable part will eventually be
+       padded up.
+       Incrementing begins before the last suffix in any case. But when this
+       rolls over on short prefixes, then long last suffixes will get used
+       as high characters of the incremental part. This is indicated by
+       *rollover_carry which corresponds to the parameter of ascii_increment()
+       with the same name.
+    */
+    static int max_suffix = 12;
+
+    *rollover_carry = 0;
+    *pre_check = 0;
+
+    len = strlen(*name);
+
+    /* Check if the part before the first dot is long enough.
+       If not, then preserve the last two short suffixes.
+    */
+    dpt = strchr(*name, '.');
+    if (dpt != NULL)
+        if ((dpt - *name) < min_incr)
+            dpt = strrchr(*name, '.');
+    if (dpt != NULL) {
+        first= (dpt - *name);
+        if (dpt > *name && len - first < max_suffix) {
+            for(npt = dpt - 1; npt >= *name && *npt != '.'; npt--);
+            if (npt >= *name) {
+                if (len - (npt - *name) <= max_suffix) {
+                    first= (npt - *name);
+                    dpt = npt;
+                }
+            }
+        }
+    } else
+        first= len;
+    if (first < min_incr && (len - first) <= max_suffix) {
+        ret = insert_underscores(*name, &len, &first, min_incr - first,
+                                 unique_name);
+        if (ret < 0)
+            goto ex;
+        *pre_check = 1; /* It might now already be unique */
+
+    } else if (len < 64) {
+        /* Insert an underscore to preserve the original name at least for the
+           first few increments
+        */
+        ret = insert_underscores(*name, &len, &first, 1, unique_name);
+        if (ret < 0)
+            goto ex;
+        *pre_check = 1;
+
+    } else {
+        LIBISO_ALLOC_MEM(*unique_name, char, len + 1);
+        memcpy(*unique_name, *name, len);
+        if (first < min_incr)
+            *rollover_carry = 1; /* Do not get caged before the dots */
+    }
+    (*unique_name)[len] = 0;
+    *low_pos = first - 1;
+    ret = 1;
+ex:;
+    return(ret);
+}
+
+static
+int make_really_unique_name(IsoDir *parent, char **name, char **unique_name,
+                            IsoNode ***pos, int flag)
+{
+    int ret, rollover_carry = 0, pre_check = 0, ascii_idx = -1, len;
+
+    ret = make_incrementable_name(name, unique_name, &ascii_idx,
+                                  &rollover_carry, &pre_check);
+    if (ret < 0)
+        goto ex;
+    len = strlen(*unique_name);
+    while (1) {
+        if (!pre_check)
+            ascii_increment(*unique_name, len, ascii_idx, !!rollover_carry);
+        else
+            pre_check = 0;
+        ret = iso_dir_exists(parent, *unique_name, pos);
+        if (ret < 0)
+            goto ex;
+        if (ret == 0)
+    break;
+    }
+    *name = *unique_name;
+    ret = ISO_SUCCESS;
+ex:;
+    if (ret < 0) {
+        LIBISO_FREE_MEM(*unique_name);
+        *unique_name = NULL;
+    }
+    return ret;
+}
+
 /**
  * Recursively add a given directory to the image tree.
  * 
@@ -750,12 +918,12 @@ int check_special(IsoImage *image, mode_t mode)
  */
 int iso_add_dir_src_rec(IsoImage *image, IsoDir *parent, IsoFileSource *dir)
 {
-    int ret;
+    int ret, dir_is_open = 0;
     IsoNodeBuilder *builder;
     IsoFileSource *file;
     IsoNode **pos;
     struct stat info;
-    char *name, *path;
+    char *name, *path, *allocated_name = NULL;
     IsoNode *new;
     enum iso_replace_mode replace;
 
@@ -771,8 +939,9 @@ int iso_add_dir_src_rec(IsoImage *image, IsoDir *parent, IsoFileSource *dir)
             ret = iso_msg_submit(image->id, ISO_NULL_POINTER, ret,
                            "Can't open dir. NULL pointer caught as dir name");
         }
-        return ret;
+        goto ex;
     }
+    dir_is_open = 1;
 
     builder = image->builder;
     
@@ -785,15 +954,16 @@ int iso_add_dir_src_rec(IsoImage *image, IsoDir *parent, IsoFileSource *dir)
             if (ret < 0) {
                 /* error reading dir */
                 ret = iso_msg_submit(image->id, ret, ret, "Error reading dir");
+                goto ex;
             }
-            break;
+    break; /* End of directory */
         }
 
         path = iso_file_source_get_path(file);
         if (path == NULL) {
             ret = iso_msg_submit(image->id, ISO_NULL_POINTER, ret, 
                                  "NULL pointer caught as file path");
-            return ret;
+            goto ex;
         }
         name = strrchr(path, '/') + 1;
 
@@ -803,6 +973,8 @@ int iso_add_dir_src_rec(IsoImage *image, IsoDir *parent, IsoFileSource *dir)
             ret = iso_file_source_lstat(file, &info);
         }
         if (ret < 0) {
+            ret = iso_msg_submit(image->id, ISO_FILE_CANT_ADD, ret,
+                                 "Error when adding file %s", path);
             goto dir_rec_continue;
         }
 
@@ -825,19 +997,25 @@ int iso_add_dir_src_rec(IsoImage *image, IsoDir *parent, IsoFileSource *dir)
 
         /* find place where to insert */
         ret = iso_dir_exists(parent, name, &pos);
-        /* TODO
-         * if (ret && replace == ISO_REPLACE_ASK) {
-         *    replace = /....
-         * }
-         */
-        
-        /* chek if we must insert or not */
-        /* TODO check for other replace behavior */
-        if (ret && (replace == ISO_REPLACE_NEVER)) { 
-            /* skip file */
-            goto dir_rec_continue;
+        if (ret) {
+            /* Resolve name collision
+               e.g. caused by fs_image.c:make_hopefully_unique_name() 
+            */
+            LIBISO_FREE_MEM(allocated_name); allocated_name = NULL;
+            ret = make_really_unique_name(parent, &name, &allocated_name, &pos,
+                                          0);
+            if (ret < 0)
+                goto ex;
+            image->collision_warnings++;
+            if (image->collision_warnings < ISO_IMPORT_COLL_WARN_MAX) {
+                ret = iso_msg_submit(image->id, ISO_IMPORT_COLLISION, 0, 
+                         "File name collision resolved with %s . Now: %s",
+                         path, name);
+                if (ret < 0)
+                    goto ex;
+            }
         }
-        
+
         /* if we are here we must insert. Give user a chance for cancel */
         if (image->report) {
             int r = image->report(image, file);
@@ -846,7 +1024,7 @@ int iso_add_dir_src_rec(IsoImage *image, IsoDir *parent, IsoFileSource *dir)
                 goto dir_rec_continue;
             }
         }
-        ret = builder->create_node(builder, image, file, &new);
+        ret = builder->create_node(builder, image, file, name, &new);
         if (ret < 0) {
             ret = iso_msg_submit(image->id, ISO_FILE_CANT_ADD, ret,
                          "Error when adding file %s", path);
@@ -882,14 +1060,17 @@ dir_rec_continue:;
         /* check for error severity to decide what to do */
         if (ret < 0) {
             ret = iso_msg_submit(image->id, ret, 0, NULL);
-            if (ret < 0) {
-                break;
-            }
+            if (ret < 0)
+                goto ex;
         }
     } /* while */
 
-    iso_file_source_close(dir);
-    return ret < 0 ? ret : ISO_SUCCESS;
+    ret = ISO_SUCCESS;
+ex:;
+    if (dir_is_open)
+        iso_file_source_close(dir);
+    LIBISO_FREE_MEM(allocated_name);
+    return ret;
 }
 
 int iso_tree_add_dir_rec(IsoImage *image, IsoDir *parent, const char *dir)
@@ -1006,6 +1187,65 @@ ex:;
     return path;
 }
 
+/* Note: No reference is taken to the found node.
+   @param flag bit0= recursion
+*/
+int iso_tree_get_node_of_block(IsoImage *image, IsoDir *dir, uint32_t block,
+                               IsoNode **found, uint32_t *next_above, int flag)
+{
+    int ret, section_count, i;
+    IsoDirIter *iter = NULL;
+    IsoNode *node;
+    IsoDir *subdir;
+    IsoFile *file;
+    struct iso_file_section *sections = NULL;
+    uint32_t na = 0;
+
+    if (dir == NULL)
+        dir = image->root;
+
+    ret = iso_dir_get_children(dir, &iter);
+    while (iso_dir_iter_next(iter, &node) == 1 ) {
+
+        if (ISO_NODE_IS_FILE(node)) {
+            file = (IsoFile *) node;
+            ret = iso_file_get_old_image_sections(file, &section_count,
+                                                  &sections, 0);
+            if (ret <= 0)
+    continue;
+            for (i = 0; i < section_count; i++) {
+                if (sections[i].block <= block &&
+                    block - sections[i].block <
+                                  (((off_t) sections[i].size) + 2047) / 2048) {
+                    *found = node;
+                    ret = 1; goto ex;
+                }
+                if ((na == 0 || sections[i].block < na) &&
+                                                     sections[i].block > block)
+                    na = sections[i].block;
+            }
+            free(sections); sections = NULL;
+        } else if (ISO_NODE_IS_DIR(node)) {
+            subdir = (IsoDir *) node;
+            ret = iso_tree_get_node_of_block(image, subdir, block, found, &na,
+                                             1);
+            if (ret != 0)
+                goto ex;
+        }
+    }
+    if (next_above != NULL && (na  > 0 || !(flag & 1)))
+        if (*next_above == 0 || *next_above > na || !(flag & 1))
+            *next_above = na;
+    ret = 0;
+ex:
+    if (sections != NULL)
+        free(sections);
+    if (iter != NULL)
+        iso_dir_iter_free(iter);
+    return ret;
+}
+
+
 /* ------------------------- tree cloning ------------------------------ */
 
 static
@@ -1196,3 +1436,84 @@ int iso_tree_clone(IsoNode *node,
     return ret;
 }
 
+
+int iso_tree_resolve_symlink(IsoImage *img, IsoSymlink *sym, IsoNode **res,
+                             int *depth, int flag)
+{
+    IsoDir *cur_dir = NULL;
+    IsoNode *n, *resolved_node;
+    char *dest, *dest_start, *dest_end;
+    int ret = 0;
+    unsigned int comp_len, dest_len;
+
+    dest = sym->dest;
+    dest_len = strlen(dest);
+
+    if (dest[0] == '/') {
+
+        /* ??? How to resolve absolute links without knowing the
+               path of the future mount point ?
+           ??? Would it be better to throw error ? 
+           I can only assume that it gets mounted at / during some stage
+           of booting.
+        */;
+
+        cur_dir = img->root;
+        dest_end = dest;
+    } else {
+        cur_dir = sym->node.parent;
+        if (cur_dir == NULL)
+            cur_dir = img->root;
+        dest_end = dest - 1;
+    }
+
+    while (dest_end < dest + dest_len) {
+        dest_start = dest_end + 1;
+        dest_end = strchr(dest_start, '/');
+        if (dest_end == NULL)
+            dest_end = dest_start + strlen(dest_start);
+        comp_len = dest_end - dest_start;
+        if (comp_len == 0 || (comp_len == 1 && dest_start[0] == '.'))
+    continue;
+        if (comp_len == 2 && dest_start[0] == '.' && dest_start[1] == '.') {
+            cur_dir = cur_dir->node.parent;
+            if (cur_dir == NULL) /* link shoots over root */
+                return ISO_DEAD_SYMLINK;
+    continue;
+        }
+
+        /* Search node in cur_dir */
+        for (n = cur_dir->children; n != NULL; n = n->next)
+            if (strncmp(dest_start, n->name, comp_len) == 0 &&
+                strlen(n->name) == comp_len)
+        break;
+        if (n == NULL)
+            return ISO_DEAD_SYMLINK;
+
+        if (n->type == LIBISO_DIR) {
+            cur_dir = (IsoDir *) n;
+        } else if (n->type == LIBISO_SYMLINK) {
+            if (*depth >= LIBISO_MAX_LINK_DEPTH)
+                return ISO_DEEP_SYMLINK;
+            (*depth)++;
+            ret = iso_tree_resolve_symlink(img, (IsoSymlink *) n,
+                                           &resolved_node, depth, 0);
+            if (ret < 0)
+                return ret;
+            if (resolved_node->type != LIBISO_DIR) {
+                n = resolved_node;
+                goto leaf_type;
+            }
+            cur_dir = (IsoDir *) resolved_node;
+        } else {
+leaf_type:;
+            if (dest_end < dest + dest_len) /* attempt to dive into file */
+                return ISO_DEAD_SYMLINK;
+            *res = n;
+            return ISO_SUCCESS;
+        }
+    }
+    *res = (IsoNode *) cur_dir;
+    return ISO_SUCCESS;
+}
+
