@@ -3,7 +3,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -277,6 +277,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
    m->appended_partitions[i]= NULL;
    m->appended_part_types[i]= 0;
  }
+ m->appended_as_gpt= 0;
  m->ascii_disc_label[0]= 0;
  m->grub2_sparc_core[0]= 0;
  memset(m->hfsp_serial_number, 0, 8);
@@ -427,6 +428,10 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
 
  m->update_flags= 0;
 
+ m->show_hfs_cmd_flag= 0;
+ m->show_hfs_cmd_count= 0;
+ m->show_hfs_cmds= NULL;
+
  m->result_line[0]= 0;
  m->result_line_counter= 0;
  m->result_page_counter= 0;
