@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -25,6 +25,11 @@
 #include <fcntl.h>
 #include <errno.h>
 
+/* O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include "xorriso.h"
 #include "xorriso_private.h"
 #include "xorrisoburn.h"
@@ -293,7 +298,7 @@ int Sectorbitmap_from_file(struct SectorbitmaP **o, char *path, char *msg,
  *os_errno= 0;
  if(msg != NULL)
    msg[0]= 0;
- fd= open(path, O_RDONLY);
+ fd= open(path, O_RDONLY | O_BINARY);
  if(fd == -1) {
    *os_errno= errno;
    if(msg != NULL) {
@@ -373,7 +378,7 @@ int Sectorbitmap_to_file(struct SectorbitmaP *o, char *path, char *info,
  unsigned char buf[40];
 
  *os_errno= 0;
- fd= open(path, O_WRONLY | O_CREAT,  S_IRUSR | S_IWUSR);
+ fd= open(path, O_WRONLY | O_CREAT | O_BINARY,  S_IRUSR | S_IWUSR);
  if(fd == -1) {
    *os_errno= errno;
    if(msg != NULL) {
@@ -590,7 +595,6 @@ int Checkmediajob_new(struct CheckmediajoB **o, int flag)
  m->report_mode= 0;
  strcpy(m->event_severity, "ALL");
  m->slow_threshold_seq= 1.0;
- m->untested_valid= 0;
  return(1);
 }
 
@@ -632,7 +636,6 @@ int Checkmediajob_copy(struct CheckmediajoB *from, struct CheckmediajoB *to,
  to->report_mode= from->report_mode;
  strcpy(to->event_severity, from->event_severity);
  to->slow_threshold_seq= from->slow_threshold_seq;
- to->untested_valid= from->untested_valid;
  return(1);
 }
 
@@ -788,18 +791,6 @@ int Xorriso_check_media_setup_job(struct XorrisO *xorriso,
        job->time_limit= num;
      else
        job->item_limit= num;
-
-#ifdef NIX
-   } else if(strncmp(argv[i], "untested=", 9) == 0) {
-     if(strcmp(argv[i] + 9, "damaged") == 0)
-       job->untested_valid= 0;
-     if(strcmp(argv[i] + 9, "undamaged") == 0 ||
-        strcmp(argv[i] + 9, "ok") == 0)
-       job->untested_valid= 1;
-     else
-       goto unknown_value;
-#endif
-
    } else if(strncmp(argv[i], "use=", 4) == 0) {
      if(strcmp(argv[i] + 4, "outdev") == 0)
        job->use_dev= 1;
@@ -1005,6 +996,7 @@ ex:;
 
 
 /* @param flag bit0= mark untested areas as valid
+               bit1= leave untested areas as they are
 */
 int Xorriso_spotlist_to_sectormap(struct XorrisO *xorriso,
                                   struct SpotlisT *spotlist,
@@ -1064,7 +1056,9 @@ int Xorriso_spotlist_to_sectormap(struct XorrisO *xorriso,
      valid= quality > xorriso->check_media_bad_limit;
      if(quality == Xorriso_read_quality_untesteD && (flag & 1))
        valid= 1;
-     else if(pass == 0 && !valid)
+     if(quality == Xorriso_read_quality_untesteD && (flag & 2))
+   continue;
+     if(pass == 0 && !valid)
    continue;
      else if(pass == 1 && valid)
    continue;
@@ -1085,7 +1079,7 @@ int Xorriso_open_job_data_to(struct XorrisO *xorriso,
 {
  if(job->data_to_path[0] == 0)
    return(2);
- job->data_to_fd= open(job->data_to_path, O_RDWR | O_CREAT,
+ job->data_to_fd= open(job->data_to_path, O_RDWR | O_CREAT | O_BINARY,
                        S_IRUSR | S_IWUSR);
  if(job->data_to_fd == -1) {
    sprintf(xorriso->info_text, "Cannot open path ");
