@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011  Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2014  Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -27,6 +27,10 @@
 #include <pwd.h>
 #include <grp.h>
 
+/* O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
 
 #include "xorriso.h"
 #include "xorriso_private.h"
@@ -54,13 +58,17 @@ int Xorriso_compare_2_contents(struct XorrisO *xorriso, char *common_adr,
  int fd1= -1, ret, r1, r2, done, wanted, i, was_error= 0, use_md5= 0;
  void *stream2= NULL;
  off_t r1count= 0, r2count= 0, diffcount= 0, first_diff= -1;
- char *respt, buf1[32*1024], buf2[32*1024], offset_text[80];
+ char *respt, *buf1= NULL, *buf2= NULL, offset_text[80];
  char disk_md5[16], iso_md5[16];
  void *ctx= NULL;
+ int buf_size= 32 * 1024;
+
+ Xorriso_alloc_meM(buf1, char, buf_size);
+ Xorriso_alloc_meM(buf2, char, buf_size);
 
  respt= xorriso->result_line;
 
- fd1= open(disk_adr, O_RDONLY);
+ fd1= open(disk_adr, O_RDONLY | O_BINARY);
  if(fd1==-1) {
    sprintf(respt, "- %s (DISK) : cannot open() : %s\n",
            disk_adr, strerror(errno));
@@ -108,7 +116,7 @@ cannot_address:;
  done= 0;
  while(!done) {
 
-   wanted= sizeof(buf1);
+   wanted= buf_size;
    if(r1count+offset+wanted>disk_size)
      wanted= disk_size-r1count-offset;
    if(r1count+wanted>bytes)
@@ -122,7 +130,7 @@ cannot_address:;
      r1+= ret;
    }
 
-   wanted= sizeof(buf2);
+   wanted= buf_size;
    if(r2count+wanted>iso_size)
      wanted= iso_size-r2count;
 /*
@@ -198,10 +206,10 @@ cannot_address:;
      xorriso->pacifier_byte_count+= r1;
      if(flag&(1<<31))
        Xorriso_pacifier_callback(xorriso, "content bytes read",
-                                 xorriso->pacifier_count, 0, "", 0);
+                                 xorriso->pacifier_count, 0, "", 8);
      else
        Xorriso_pacifier_callback(xorriso, "bytes", xorriso->pacifier_count, 0,
-                                 "", 1<<6);
+                                 "", 8 | 1<<6);
    }
  }
 
@@ -248,6 +256,8 @@ cannot_address:;
 ex:;
  if(ctx != NULL)
    Xorriso_md5_end(xorriso, &ctx, disk_md5, 0);
+ Xorriso_free_meM(buf1);
+ Xorriso_free_meM(buf2);
  return(ret);
 }
 
@@ -300,7 +310,7 @@ int Xorriso_compare_2_files(struct XorrisO *xorriso, char *disk_adr,
  int ret, missing= 0, is_split= 0, i, was_error= 0, diff_count= 0;
  int content_shortcut= 0, mask;
  char *respt;
- char a[5*SfileadrL], sfe[5*SfileadrL];
+ char *a= NULL;
  char ttx1[40], ttx2[40];
  char *a1_acl= NULL, *a2_acl= NULL, *d1_acl= NULL, *d2_acl= NULL;
  char *attrlist1= NULL, *attrlist2= NULL;
@@ -308,18 +318,22 @@ int Xorriso_compare_2_files(struct XorrisO *xorriso, char *disk_adr,
  int split_count= 0;
  time_t stamp;
 
- char part_path[SfileadrL], *part_name;
+ char *part_path= NULL, *part_name;
  int partno, total_parts= 0;
  off_t offset, bytes, total_bytes;
 
+ Xorriso_alloc_meM(a, char, 5*SfileadrL);
+ Xorriso_alloc_meM(part_path, char, SfileadrL);
+ 
  *result= 0;
  respt= xorriso->result_line;
 
  if(!(xorriso->disk_excl_mode&8)) {
    ret= Xorriso_path_is_excluded(xorriso, disk_adr, 2 | !!(flag&(1<<27)));
    if(ret>0) {
-     sprintf(respt , "? %s (DISK) : exluded by %s\n",
-             Text_shellsafe(disk_adr, sfe, 0),
+     strcpy(respt, "? ");
+     Text_shellsafe(disk_adr, respt, 1);
+     sprintf(respt + strlen(respt), " (DISK) : exluded by %s\n",
              (ret==1 ? "-not_paths" : "-not_leaf"));
      if(! (flag & ((1 << 31) | (1 << 26))))
        Xorriso_result(xorriso,0);
@@ -333,8 +347,10 @@ int Xorriso_compare_2_files(struct XorrisO *xorriso, char *disk_adr,
    else
      ret= lstat(disk_adr, &s1);
    if(ret==-1) {
-     sprintf(respt , "? %s (DISK) : cannot lstat() : %s\n",
-             Text_shellsafe(disk_adr, sfe, 0), strerror(errno));
+     strcpy(respt, "? ");
+     Text_shellsafe(disk_adr, respt, 1);
+     sprintf(respt + strlen(respt),
+             " (DISK) : cannot lstat() : %s\n", strerror(errno));
      if(! (flag & ((1 << 31) | (1 << 26))))
        Xorriso_result(xorriso,0);
      missing= 1;
@@ -347,7 +363,7 @@ int Xorriso_compare_2_files(struct XorrisO *xorriso, char *disk_adr,
    strcpy(a, Ftypetxt(s1.st_mode, 1));  
  strcat(a, " ");
  if(adr_common_tail[0])
-   strcat(a, Text_shellsafe(adr_common_tail, sfe, 0));
+   Text_shellsafe(adr_common_tail, a, 1);
  else {
    Text_shellsafe(disk_adr, a+strlen(a), 0);
    strcat(a, " (DISK)");
@@ -361,8 +377,9 @@ int Xorriso_compare_2_files(struct XorrisO *xorriso, char *disk_adr,
 
  ret= Xorriso_iso_lstat(xorriso, iso_adr, &s2, 0);
  if(ret<0) {
-   sprintf(respt, "? %s  (ISO) : cannot find this file in ISO image\n",
-           Text_shellsafe(iso_adr, sfe, 0));
+   strcpy(respt, "? ");
+   Text_shellsafe(iso_adr, respt, 1);
+   strcat(respt, "  (ISO) : cannot find this file in ISO image\n");
    if(!(flag&(1<<31)))
      Xorriso_result(xorriso,0);
    missing= 1;
@@ -652,6 +669,8 @@ ex:;
    free(a2_acl);
  if(d2_acl != NULL)
    free(d2_acl);
+ Xorriso_free_meM(part_path);
+ Xorriso_free_meM(a);
  return(ret);
 }
 
@@ -661,20 +680,25 @@ int Xorriso_pfx_disk_path(struct XorrisO *xorriso, char *iso_path,
                           char disk_path[SfileadrL], int flag)
 {
  int ret;
- char adrc[SfileadrL];
+ char *adrc= NULL;
+
+ Xorriso_alloc_meM(adrc, char, SfileadrL);
 
  if(strncmp(iso_path, iso_prefix, strlen(iso_prefix))!=0)
-   return(-1);
+   {ret= -1; goto ex;}
  if(strlen(disk_prefix) + strlen(iso_path) - strlen(iso_prefix)+1 >= SfileadrL)
-   return(-1);
+   {ret= -1; goto ex;}
  if(iso_path[strlen(iso_prefix)] == '/')
    strcpy(adrc, iso_path + strlen(iso_prefix) + 1);
  else
    strcpy(adrc, iso_path + strlen(iso_prefix));
  ret= Xorriso_make_abs_adr(xorriso, disk_prefix, adrc, disk_path, 4 | 8);
  if(ret <= 0)
-   return(ret);
- return(1);
+   goto ex;
+ ret= 1;
+ex:;
+ Xorriso_free_meM(adrc);
+ return(ret);
 }
 
 
@@ -691,12 +715,14 @@ int Xorriso_find_compare(struct XorrisO *xorriso, void *boss_iter, void *node,
                          int flag)
 {
  int ret, result, uret, follow_links, deleted= 0;
- char disk_path[SfileadrL];
+ char *disk_path= NULL;
+
+ Xorriso_alloc_meM(disk_path, char, SfileadrL);
 
  ret= Xorriso_pfx_disk_path(xorriso, iso_path, iso_prefix, disk_prefix,
                             disk_path, 0);
  if(ret <= 0)
-   return(ret);
+   goto ex;
 
  /* compare exclusions against disk_path resp. leaf name */
  if(xorriso->disk_excl_mode&8)
@@ -704,9 +730,9 @@ int Xorriso_find_compare(struct XorrisO *xorriso, void *boss_iter, void *node,
  else
    ret= 0;
  if(ret<0)
-   return(ret);
+   goto ex;
  if(ret>0)
-   return(3);
+   {ret= 3; goto ex;}
 
  follow_links= (xorriso->do_follow_links ||
                (xorriso->do_follow_param && !(flag&2))) <<28;
@@ -719,7 +745,7 @@ int Xorriso_find_compare(struct XorrisO *xorriso, void *boss_iter, void *node,
  if(flag&1) {
    if(ret<0)
      if(Xorriso_eval_problem_status(xorriso, ret, 1|2)<0)
-       return(ret);
+       goto ex;
    if(ret > 0)
      result= 0;
    uret= Xorriso_update_interpreter(xorriso, boss_iter, node, result,
@@ -731,11 +757,13 @@ int Xorriso_find_compare(struct XorrisO *xorriso, void *boss_iter, void *node,
      deleted= 1;
  }
  if(ret<0)
-   return(ret);
+   goto ex;
  if(deleted)
-   return(2);
+   {ret= 2; goto ex;}
  if(result&(1<<17))
-   return(3);
+   {ret= 3; goto ex;}
+ex:;
+ Xorriso_free_meM(disk_path);
  return(ret);
 }
 
@@ -759,22 +787,23 @@ int Xorriso_update_interpreter(struct XorrisO *xorriso,
                                char *iso_rr_path, int flag)
 {
  int ret= 1, deleted= 0, is_split= 0, i, loop_count, late_hardlink_update= 0;
- char sfe[5*SfileadrL];
  struct stat stbuf;
  struct SplitparT *split_parts= NULL;
  int split_count= 0;
- char part_path[SfileadrL], *part_name;
+ char *part_path= NULL, *part_name;
  int partno, total_parts, new_total_parts;
  off_t offset, bytes, total_bytes, disk_size, first_bytes;
 
  if((compare_result&3)==3) {
-   sprintf(xorriso->info_text, "Missing on disk and in ISO: disk_path %s",
-           Text_shellsafe(disk_path, sfe, 0));
+   sprintf(xorriso->info_text, "Missing on disk and in ISO: disk_path ");
+   Text_shellsafe(disk_path, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 1);
    xorriso->find_compare_result= -1;
    ret= 3; goto ex;
  }
 
+ Xorriso_alloc_meM(part_path, char, SfileadrL);
+
  if((flag & 2) && !(compare_result & 2)) {
    ret= Xorriso_mark_update_merge(xorriso, iso_rr_path, node,
                                   !(compare_result & 1));
@@ -788,8 +817,8 @@ int Xorriso_update_interpreter(struct XorrisO *xorriso,
    if(flag & 1)
      {ret= 3; goto ex;}
    /* cannot open regular disk file, early eof of disk file */
-   sprintf(xorriso->info_text, "Problems with reading disk file %s",
-           Text_shellsafe(disk_path, sfe, 0));
+   sprintf(xorriso->info_text, "Problems with reading disk file ");
+   Text_shellsafe(disk_path, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 1);
    xorriso->find_compare_result= -1;
    ret= 1; goto ex;
@@ -971,13 +1000,14 @@ overwrite:;
  } else
    ret= 1;
  if(ret>0 && xorriso->info_text[0]) {
-   strcat(xorriso->info_text, Text_shellsafe(iso_rr_path, sfe, 0));
+   Text_shellsafe(iso_rr_path, xorriso->info_text, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "UPDATE", 0);
  }
  ret= 1;
 ex:;
  if(split_parts!=NULL)
    Splitparts_destroy(&split_parts, split_count, 0);
+ Xorriso_free_meM(part_path);
  if(ret<=0)
    return(ret);
  if(deleted)
