@@ -112,6 +112,9 @@ int Xorriso_make_mount_cmd(struct XorrisO *xorriso, char *cmd,
 
 int Xorriso_append_scdbackup_record(struct XorrisO *xorriso, int flag);
 
+int Xorriso_concat(struct XorrisO *xorriso, char *mode, char *target,
+                   int progc, char **progv,
+                   int filec, char **filev, int flag);
 
 #endif /* ! Xorriso_pvt_diskop_includeD */
 
