@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -598,6 +598,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "--no_rc", "--norock", "-hfsplus", "-fat", "-chrp-boot-part",
     "-isohybrid-gpt-basdat", "-isohybrid-gpt-hfsplus",
     "-isohybrid-apm-hfsplus", "--grub2-boot-info", "-joliet-utf16",
+    "-appended_part_as_gpt",
     ""
  };
  static char arg1_options[][41]= {
@@ -625,6 +626,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "-volid", "-old-exclude",
     "-alpha-boot", "-hppa-cmdline", "-hppa-kernel-32", "-hppa-kernel-64",
     "-hppa-bootloader", "-hppa-ramdisk", "-mips-boot", "-mipsel-boot",
+    "-alpha-boot",
     "-jigdo-jigdo", "-jigdo-template", "-jigdo-min-file-size",
     "-jigdo-force-md5", "-jigdo-exclude", "-jigdo-map", "-md5-list",
     "-jigdo-template-compress",
@@ -860,13 +862,16 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -hppa-bootloader FILE       Set hppa boot loader file name (relative to image root)",
 "  -hppa-ramdisk FILE          Set hppa ramdisk file name (relative to image root)",
 "  -hppa-hdrversion NUMBER     Set hppa PALO header version to 4 or 5",
+"  -alpha-boot FILE            Set alpha boot image name (relative to image root)",
 "  --grub2-sparc-core FILE     Set path of core file for disk label patching",
 "  -efi-boot-part DISKFILE|--efi-boot-image",
 "                              Set data source for EFI System Partition",
-"  -chrp-boot-part             Mark ISO image size by MBR partition type 0x41",
-"  -prep-boot-part DISKFILE    Set data source for MBR partition type 0x96",
+"  -chrp-boot-part             Mark ISO image size by MBR partition type 0x96",
+"  -chrp-boot                  Alias of -chrp-boot-part",
+"  -prep-boot-part DISKFILE    Set data source for MBR partition type 0x41",
 "  -append_partition NUMBER TYPE FILE",
 "                              Append FILE after image. TYPE is hex: 0x..",
+"  -appended_part_as_gpt       mark appended partitions in GPT instead of MBR.",
 "  --modification-date=YYYYMMDDhhmmsscc",
 "                              Override date of creation and modification",
 "  -isohybrid-mbr FILE         Set SYSLINUX mbr/isohdp[fp]x*.bin for isohybrid",
@@ -1182,7 +1187,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
  char *old_root= NULL, *argpt, *hargv[1];
  char *boot_path, partno_text[8], *iso_rr_pt, *disk_pt, *rpt, *wpt;
  char *rm_merge_args[3], *rr_reloc_dir_pt= NULL;
- char *sort_weight_args[4], *bless_args[6];
+ char *sort_weight_args[4], *bless_args[6], *sa_path;
 
  struct stat stbuf;
 
@@ -1794,7 +1799,8 @@ not_enough_args:;
              strcmp(argpt, "-partition_cyl_align") == 0 ||
              strcmp(argpt, "-isohybrid-mbr") == 0 ||
              strcmp(argpt, "--grub2-mbr") == 0 ||
-             strncmp(argpt, "-hppa-", 6) == 0) {
+             strncmp(argpt, "-hppa-", 6) == 0 ||
+             strcmp(argpt, "-alpha-boot") == 0) {
      if(i+1>=argc)
        goto not_enough_args;
      delay_opt_list[delay_opt_count++]= i;
@@ -1930,7 +1936,8 @@ not_enough_args:;
      if(ret <= 0)
        goto problem_handler_2;
 
-   } else if(strcmp(argpt, "-chrp-boot-part") == 0) {
+   } else if(strcmp(argpt, "-chrp-boot-part") == 0 ||
+             strcmp(argpt, "-chrp-boot") == 0) {
      ret= Xorriso_option_boot_image(xorriso, "any", "chrp_boot_part=on", 0);
      if(ret <= 0)
        goto problem_handler_2;
@@ -1960,6 +1967,9 @@ not_enough_args:;
      if(ret <= 0)
        goto problem_handler_2;
 
+   } else if(strcmp(argpt, "-appended_part_as_gpt") == 0) {
+     xorriso->appended_as_gpt= 1;
+
    } else if(strcmp(argpt, "-B") == 0 ||
              strcmp(argpt, "-sparc-boot") == 0) {
      i++;
@@ -2012,8 +2022,10 @@ not_enough_args:;
      if(i+1>=argc)
        goto not_enough_args;
      i++;
-     strncpy(xorriso->ascii_disc_label, argv[i], Xorriso_disc_label_sizE - 1);
-     xorriso->ascii_disc_label[Xorriso_disc_label_sizE - 1] = 0;
+     sprintf(sfe, "sparc_label=%s", argv[i]);
+     ret= Xorriso_option_boot_image(xorriso, "any", sfe, 0);
+     if(ret <= 0)
+       goto problem_handler_2;
 
    } else if(strcmp(argpt, "--grub2-sparc-core") == 0) {
      if(i+1>=argc)
@@ -2363,7 +2375,7 @@ problem_handler_2:;
      if(ret <= 0)
        goto problem_handler_boot;
    } else if(strcmp(argpt, "-isohybrid-gpt-basdat") == 0) {
-     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x1fc) |
+     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x0fc) |
                                      (1 << 2);
    } else if(strcmp(argpt, "-isohybrid-gpt-hfsplus") == 0) {
      xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x0fc) |
@@ -2382,7 +2394,11 @@ problem_handler_2:;
      if(i+1>=argc)
        goto not_enough_args;
      i++; 
-     ret= Xorriso_set_system_area_path(xorriso, argv[i], 0);
+     if(strcmp(argv[i], ".") == 0)
+       sa_path= "";
+     else
+       sa_path= argv[i];
+     ret= Xorriso_set_system_area_path(xorriso, sa_path, 0);
      if(ret <= 0)
        goto problem_handler_boot;
      if(strcmp(argpt, "-isohybrid-mbr")==0)
@@ -2439,6 +2455,16 @@ problem_handler_2:;
      if(ret <= 0)
        goto problem_handler_boot;
 
+   } else if(strcmp(argpt, "-alpha-boot") == 0) {
+     if(i + 1 >= argc)
+       goto not_enough_args;
+     i++;
+     sprintf(sfe, "-as mkisofs %s %s", argpt, argv[i]);
+     ret= Xorriso_coordinate_system_area(xorriso, 6, 0, sfe, 0);
+     if(ret <= 0)
+       goto ex;
+     ret= Xorriso_set_alpha_boot(xorriso, argv[i], 0);
+
    } else if(strcmp(argpt, "-hfs-bless") == 0) {
      static char *bless_arg_data[6]= {
         "/", "-disk_path", "", "-exec", "set_hfs_bless", "p"};
