@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -172,6 +172,8 @@ struct FindjoB {
    49= internal: update creator, type, and blessings from persistent isofs.*
    50= print_outname namespace
    51= report_sections
+   52= show_stream_id
+   53= internal: show_hfs_cmd , controlled by xorriso->show_hfs_cmd*
  */
  int action;
  int prune;
