@@ -2220,6 +2220,8 @@ int Xorriso_retrieve_disk_path(struct XorrisO *xorriso, IsoNode *node,
 }
 
 
+/* @param flag bit0= show numbers from iso_stream_get_id
+ */
 int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
                         char *path, int flag)
 {
@@ -2229,6 +2231,9 @@ int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
  IsoStream *stream= NULL, *input_stream;
  IsoExternalFilterCommand *cmd;
  char type_text[16], *source_path= NULL;
+ unsigned int fs_id;
+ dev_t dev_id;
+ ino_t ino_id;
 
  node= (IsoNode *) in_node;
  if(node == NULL) {
@@ -2249,6 +2254,12 @@ int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
    strcat(xorriso->result_line, " < ");
    Xorriso_stream_type(xorriso, node, stream, type_text, 0);
    strcat(xorriso->result_line, type_text);
+   if(flag & 1) {
+     iso_stream_get_id(stream, &fs_id, &dev_id, &ino_id);
+     sprintf(xorriso->result_line + strlen(xorriso->result_line),
+             "[%u,%lu,%lu]", fs_id, (unsigned long) dev_id,
+                             (unsigned long) ino_id);
+   }
    ret= iso_stream_get_external_filter(stream, &cmd, 0);
    if(ret < 0) {
      Xorriso_process_msg_queues(xorriso,0);
@@ -2268,6 +2279,11 @@ int Xorriso_show_stream(struct XorrisO *xorriso, void *in_node,
  strcat(xorriso->result_line, " < ");
  Xorriso_stream_type(xorriso, node, stream, type_text, 0);
  strcat(xorriso->result_line, type_text);
+ if(flag & 1) {
+   iso_stream_get_id(stream, &fs_id, &dev_id, &ino_id);
+   sprintf(xorriso->result_line + strlen(xorriso->result_line), "[%u,%lu,%lu]",
+           fs_id, (unsigned long) dev_id, (unsigned long) ino_id);
+ }
 
  source_path= iso_stream_get_source_path(stream, 0);
  if(source_path != NULL) {
