@@ -1,11 +1,11 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
-   This file contains the implementation of options -a* to -c* as mentioned
+   This file contains the implementation of commands -a* to -c* as mentioned
    in man page or info file derived from xorriso.texi.
 */
 
@@ -716,7 +716,7 @@ treatment_patch:;
 
  } else if(strcmp(treatpt, "discard")==0) {
    xorriso->keep_boot_image= 0;
-   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
+   xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~0x3ff) | 0;
    xorriso->boot_image_bin_path[0]= 0;
    xorriso->patch_system_area= 0;
    if((xorriso->system_area_options & 0xfc ) == 0)
@@ -866,10 +866,12 @@ treatment_patch:;
  } else if(strcmp(treatpt, "mips_discard") == 0 ||
            strcmp(treatpt, "mipsel_discard") == 0 ||
            strcmp(treatpt, "sparc_discard") == 0 || 
-           strcmp(treatpt, "hppa_discard") == 0) {
+           strcmp(treatpt, "hppa_discard") == 0 ||
+           strcmp(treatpt, "alpha_discard") == 0) {
    xorriso->system_area_options&= ~0xfc; /* system area type 0 */
    Xorriso_add_mips_boot_file(xorriso, "", 1); /* give up MIPS boot files */
    Xorriso_set_hppa_boot_parm(xorriso, "", "", 1); /* give up HP-PA files */
+   Xorriso_set_alpha_boot(xorriso, "", 1); /* give up DEC Alpha loader */
 
  } else if(strncmp(treatpt, "sparc_label=", 12) == 0) {
    sprintf(eff_path, "-boot_image %s sparc_label=", formpt);
@@ -909,6 +911,13 @@ treatment_patch:;
    parm[parm_len]= 0;
    ret= Xorriso_set_hppa_boot_parm(xorriso, eqpt + 1, parm, 0);
 
+ } else if(strncmp(treatpt, "alpha_boot=", 11) == 0) {
+   sprintf(eff_path, "-boot_image %s %s", formpt, treatpt);
+   ret= Xorriso_coordinate_system_area(xorriso, 6, 0, eff_path, 0);
+   if(ret <= 0)
+     goto ex;
+   ret = Xorriso_set_alpha_boot(xorriso, treatpt + 11, 0);
+
  } else if(strncmp(treatpt, "boot_info_table=", 16)==0) {
    if(strcmp(treatpt + 16, "off") == 0)
      xorriso->patch_isolinux_image= (xorriso->patch_isolinux_image & ~3) | 0;
@@ -965,7 +974,11 @@ treatment_patch:;
      if(ret <= 0)
        goto ex;
    }
-   ret= Xorriso_set_system_area_path(xorriso, treatpt + 12, 0);
+   if(strcmp(treatpt + 12, ".") == 0) {
+     ret= Xorriso_set_system_area_path(xorriso, "", 0);
+   } else {
+     ret= Xorriso_set_system_area_path(xorriso, treatpt + 12, 0);
+   }
    if(ret <= 0)
      goto ex;
    xorriso->system_area_options&= ~0x4000;
@@ -983,6 +996,26 @@ treatment_patch:;
        goto ex;
    } else
      was_ok= 0;
+ } else if(strncmp(treatpt, "partition_entry=", 16)==0) {
+   if(strcmp(formpt, "isolinux") != 0) {
+     sprintf(xorriso->info_text,
+        "-boot_image %s partition_entry=%s : Wrong type. Need \"isolinux\".",
+             formpt, treatpt + 16);
+     Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
+     {ret= 0; goto ex;}
+   }
+   if(strcmp(treatpt + 16, "gpt_basdat") == 0) {
+     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x0fc) |
+                                     (1 << 2);
+   } else if(strcmp(treatpt + 16, "gpt_hfsplus") == 0) {
+     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x0fc) |
+                                     (2 << 2);
+   } else if(strcmp(treatpt + 16, "apm_hfsplus") == 0) {
+     xorriso->patch_isolinux_image = xorriso->patch_isolinux_image | (1 << 8);
+   } else if(strcmp(treatpt + 16, "off") == 0) {
+     xorriso->patch_isolinux_image = (xorriso->patch_isolinux_image & ~0x1fc);
+   } else
+     was_ok= 0;
 
  } else if(strncmp(treatpt, "partition_offset=", 17)==0) {
    u= 0;
@@ -996,6 +1029,14 @@ treatment_patch:;
    }
    xorriso->partition_offset= u;
 
+ } else if(strncmp(treatpt, "appended_part_as=", 17) == 0) {
+   if(strcmp(treatpt + 17, "gpt") == 0) {
+     xorriso->appended_as_gpt = 1;
+   } else if(strcmp(treatpt + 17, "mbr") == 0) {
+     xorriso->appended_as_gpt = 0;
+   } else
+     was_ok= 0;
+
  } else if(strncmp(treatpt, "partition_hd_cyl=", 17)==0) {
    u= 0;
    sscanf(treatpt + 17, "%u", &u);
@@ -1039,6 +1080,7 @@ treatment_patch:;
    }
 
  } else if(strncmp(treatpt, "platform_id=", 12)==0) {
+   u= 256; /* intentionally too large */
    if(strncmp(treatpt + 12, "0x", 2) == 0)
      sscanf(treatpt + 14, "%x", &u);
    else
@@ -1180,7 +1222,11 @@ treatment_patch:;
    ret= Xorriso_coordinate_system_area(xorriso, 0, (1 << 14), eff_path, 1 | 2);
    if(ret <= 0)
      goto ex;
-   ret= Xorriso_set_system_area_path(xorriso, treatpt + 10, 0);
+   if(strcmp(treatpt + 10, ".") == 0) {
+     ret= Xorriso_set_system_area_path(xorriso, "", 0);
+   } else {
+     ret= Xorriso_set_system_area_path(xorriso, treatpt + 10, 0);
+   }
    if(ret <= 0)
      goto ex;
    if(treatpt [10] == 0)
