@@ -1,11 +1,11 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
-   This file contains the implementation of options as mentioned in man page
+   This file contains the implementation of commands as mentioned in man page
    or info file derived from xorriso.texi.
 */
 
@@ -770,6 +770,8 @@ not_enough_arguments:;
      if(i+2>=end_idx)
        goto not_enough_arguments;
      i+= 2;
+
+     /* >>> if letter suffix: use Scanf_io_size */
      sscanf(argv[i-1], "%d", &start_lba);
      sscanf(argv[i], "%d", &count);
      Findjob_set_lba_range(job, start_lba, count, 0);
@@ -1225,6 +1227,8 @@ not_enough_exec_arguments:;
        Findjob_set_action_type(job, 50, name_space, 0);
      } else if(strcmp(cpt, "report_sections")==0) {
        Findjob_set_action_target(job, 51, NULL, 0);
+     } else if(strcmp(cpt, "show_stream_id") == 0) {
+       Findjob_set_action_target(job, 52, NULL, 0);
      } else {
        sprintf(xorriso->info_text, "-find -exec: unknown action ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
@@ -1732,15 +1736,19 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              |\"discard\"|\"keep\"|\"patch\"|\"dir=\"|\"bin_path=\"",
 "              |\"cat_path=\"|\"cat_hidden=on|iso_rr|joliet|off\"",
 "              |\"load_size=\"|\"boot_info_table=\"",
-"              |\"grub2_boot_info=\"|\"grub2_mbr=\"",
+"              |\"grub2_boot_info=\"|\"grub2_mbr=\"|\"partition_offset=\"",
+"              |\"partition_hd_cyl=\"|\"partition_sec_hd=\"",
+"              |\"partition_cyl_align=\"",
 "              |\"system_area=\"|\"partition_table=on|off\"",
+"              |\"partition_entry=\"|\"appended_part_as=\"",
 "              |\"chrp_boot_part=on|off=\"|\"prep_boot_part=\"",
 "              |\"efi_boot_part=\"|\"efi_boot_part=--efi-boot-image\"",
 "              |\"mips_path=\"|\"mipsel_path=\"|\"mips_discard\"",
 "              |\"sparc_label=\"|\"grub2_sparc_core=\"|\"sparc_discard\"",
 "              |\"hppa_cmdline=\"|\"hppa_bootloader=\"|\"hppa_kernel_32=\"",
 "              |\"hppa_kernel_64=\"|\"hppa_ramdisk=\"|\"hppa_hdrversion=\"",
-"              |\"hppa_discard\"|\"hfsplus_serial=\"|\"hfsplus_block_size=\"",
+"              |\"hppa_discard\"|\"alpha_boot=\"|\"alpha_discard\"",
+"              |\"hfsplus_serial=\"|\"hfsplus_block_size=\"",
 "              |\"apm_block_size=\"|\"show_status\"",
 "                 Whether to discard or keep an exiting El Torito boot image.",
 "                 ISOLINUX can be made bootable by dir=/ or dir=/isolinux",
@@ -1755,7 +1763,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 mips_path= adds Big Endian MIPS boot files. mipsel_path=",
 "                 sets one Little Endian MIPS boot file. sparc_label=",
 "                 activates SUN Disk Label. hppa_* is for HP PA-RISC via PALO.",
-"                 All four are mutually exclusive and exclusive to production",
+"                 alpha_boot= is for DEC Alpha SRM. MIPS, SUN, HP, and Alpha",
+"                 are mutually exclusive and exclusive to production",
 "                 of MBR and to booting via EFI from USB stick.",
 "  -append_partition partition_number type_code disk_path",
 "                 Append a prepared filesystem image after the end of the",
@@ -1915,12 +1924,13 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   -if, -then, -elseif, -else, -endif",
 "                 Action may be one of: echo, chown, chown_r, chgrp, chgrp_r",
 "                   chmod, chmod_r, alter_date, alter_date_r, lsdl, compare,",
-"                   rm, rm_r, compare, update, report_damage, report_lba,",
+"                   rm, rm_r, compare, update, report_damage,",
+"                   report_lba, report_sections,",
 "                   getfacl, setfacl, getfattr, setfattr, get_any_xattr,",
 "                   list_extattr, get_md5, check_md5, make_md5,",
 "                   set_hfs_crtp, get_hfs_crtp, set_hfs_bless, get_hfs_bless,",
-"                   set_filter, show_stream, mkisofs_r, hide, print_outname,",
-"                   estimate_size, find",
+"                   set_filter, show_stream, show_stream_id, mkisofs_r,",
+"                   hide, print_outname, estimate_size, find",
 "                 params are their parameters except iso_rr_path.",
 "  -mkdir iso_rr_path [...]",
 "                 Create empty directories if they do not exist yet.",
