@@ -146,7 +146,7 @@ int Xorriso__get_di(IsoNode *node, dev_t *dev, ino_t *ino, int flag)
 {
  int ret, i, i_end, imgid, error_code;
  size_t value_length= 0;
- char *value= NULL, msg[ISO_MSGS_MESSAGE_LEN], severity[80];
+ char *value= NULL, *msg= NULL, severity[80];
  unsigned char *vpt;
  static char *name= "isofs.di";
 
@@ -155,22 +155,39 @@ int Xorriso__get_di(IsoNode *node, dev_t *dev, ino_t *ino, int flag)
  Xorriso_get_di_counteR++;
 #endif /* NIX */
 
+ msg= TSOB_FELD(char, ISO_MSGS_MESSAGE_LEN);
+ if(msg == NULL)
+   {ret= -1; goto ex;}
  *dev= 0;
  *ino= 0;
  ret= iso_node_lookup_attr(node, name, &value_length, &value, 0);
  if(ret <= 0) {
    /* Drop any pending messages because there is no xorriso to take them */
    iso_obtain_msgs("NEVER", &error_code, &imgid, msg, severity);
-   return(ret);
+   goto ex;
  }
  vpt= (unsigned char *) value;
- for(i= 1; i <= vpt[0] && i < value_length; i++)
+ if(vpt[0] > sizeof(ino_t)) {
+
+   /* >>> obviously not the same system that recorded the device number */; 
+
+ }
+ for(i= 1; i <= vpt[0] && i < (int) value_length; i++)
    *dev= ((*dev) << 8) | vpt[i];
  i_end= i + vpt[i] + 1;
- for(i++; i < i_end && i < value_length; i++)
+ if(vpt[i] > sizeof(ino_t)) {
+
+   /* >>> obviously not the same system that recorded the inode number */; 
+
+ }
+ for(i++; i < i_end && i < (int) value_length; i++)
    *ino= ((*ino) << 8) | vpt[i];
  free(value);
- return(1);
+ ret= 1;
+ex:;
+ if(msg != NULL)
+   free(msg);
+ return(ret);
 }
 
 
@@ -639,9 +656,9 @@ int Xorriso_hardlink_update(struct XorrisO *xorriso, int *compare_result,
    "xorriso_DEBUG: old_idx= %d , low= %d , high= %d , iso= '%s' , disk='%s'\n",
    old_idx, low, high, iso_rr_path, disk_path);
    fprintf(stderr,
-         "xorriso_DEBUG: old_dev= %lu , old_ino= %lu ,  dev= %lu , ino= %lu\n",
-         (unsigned long) old_dev, (unsigned long) old_ino,
-         (unsigned long) stbuf.st_dev, (unsigned long) stbuf.st_ino);
+         "xorriso_DEBUG: old_dev= %lu , old_ino= %.f ,  dev= %lu , ino= %.f\n",
+         (unsigned long) old_dev, (double) old_ino,
+         (unsigned long) stbuf.st_dev, (double) stbuf.st_ino);
 
    if(idx >= 0 && idx != old_idx)
      fprintf(stderr, "xorriso_DEBUG: idx= %d , old_idx = %d\n", idx, old_idx);
@@ -660,8 +677,8 @@ int Xorriso_hardlink_update(struct XorrisO *xorriso, int *compare_result,
  dev_t dev;
 
  Xorriso__get_di(xorriso->di_array[i], &dev, &ino, 0);
- fprintf(stderr, "xorriso_DEBUG: iso_sibling= '%s' , dev= %lu , ino= %lu\n",
-         node_path, (unsigned long) dev, (unsigned long) ino);
+ fprintf(stderr, "xorriso_DEBUG: iso_sibling= '%s' , dev= %lu , ino= %.f\n",
+         node_path, (unsigned long) dev, (double) ino);
 }
 #endif /* Xorriso_hardlink_update_debuG */
 
