@@ -2,7 +2,7 @@
 /* Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -441,7 +441,8 @@ struct XorrisO { /* the global context of xorriso */
  int patch_system_area;               /* Bits as of system_area_options.
                                          to be applied to the loaded system
                                          area of the image, if no 
-                                         system_area_disk_path is set.
+                                         system_area_disk_path and no
+                                         system_area_options are set.
                                        */
 
  /* The number of unclaimed 2K blocks before start of partition 1 as of
@@ -461,6 +462,9 @@ struct XorrisO { /* the global context of xorriso */
  /* Path and type of image files to be appended as MBR partitions */
  char *appended_partitions[Xorriso_max_appended_partitionS];
  uint8_t appended_part_types[Xorriso_max_appended_partitionS];
+ /* If 1: With appended partitions: create protective MBR and mark by GPT */
+ int appended_as_gpt;
+
 
  /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
  */
@@ -754,6 +758,15 @@ struct XorrisO { /* the global context of xorriso */
  */
  int update_flags;
 
+ /* For find jobs show_hfs_cmd , show_hfs_as_mkisofs 
+ */
+ /* bit0= do not record but only count
+    bit1= as_mkisofs mode
+ */
+ int show_hfs_cmd_flag;
+ int show_hfs_cmd_count;
+ char **show_hfs_cmds;
+
  /* result (stdout, R: ) */
  char result_line[10*SfileadrL];
  int result_line_counter;
